<?php
/**
 * Webfolio CMS
 *
 * Copyright (C) 2009 Nikola Posa (http://www.nikolaposa.in.rs)
 *
 * This file is part of Webfolio CMS.
 *
 * Webfolio CMS is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 3 of the License, or 
 * (at your option) any later version.
 *
 * Webfolio CMS is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with Webfolio CMS. If not, see <http://www.gnu.org/licenses/>.
 */
 
/**
 * Base model that provides array access to object, and 
 * should be extended by all Webfolio CMS ORM models.
 *
 * @author Nikola Posa <posa.nikola@gmail.com>
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License
 */
abstract class Model_Base implements ArrayAccess
{
	/**
	 * Data mapper of this model.
	 *
	 * @var object
	 */
	protected $mapper;

    /**
     * Filter used for filtering name in __get() and __set() calls.
     * 
     * @var Zend_Filter_Interface
     */
    protected static $_offsetFilter;
	
	/**
	 * Constructor.
	 *
	 * @param array Options for object's initialization.
	 * @return Model_Base
	 */
	public function __construct(array $options = null)
    {
        if (is_array($options)) {
            $this->setOptions($options);
        }
    }

    /**
     * Gets offset filter instance.
     *
     * @return Zend_Filter_Interface
     */
    protected static function _getOffsetFilter()
    {
        if (!self::$_offsetFilter) {
            self::$_offsetFilter = new Zend_Filter_Word_UnderscoreToCamelCase();
        }

        return self::$_offsetFilter;
    }
	
	/**
	 * Allows access to properties of this class.
	 * 
	 * @param string Name of the property.
	 * @param mixed Value that will be set.
	 * @return void
	 */
	public function __set($name, $value)
    {		
		$name = $this->filterOffset($name);
		
        $method = 'set' . ucfirst($name);
        if (method_exists($this, $method)) { //Only if property exists.
             $this->$method($value);
        }
		else {
			$this->$name = $value;
		}
    }
	
	/**
	 * Allows access to properties of this class.
	 * 
	 * @param string Name of the property.
	 * @return mixed
	 */
	public function __get($name)
    {
		$name = $this->filterOffset($name);
		
        $method = 'get' . ucfirst($name);
        if (method_exists($this, $method)) { //Only if property exists.
            return $this->$method();
        }
		else {
			return $this->$name;
		}
    }
	
	/**
	 * Filters offset, so it becomes camel-cased.
	 *
	 * @param string Offset.
	 * @return string
	 */
	protected function filterOffset($offset)
	{
		if (strpos($offset, '_') !== false) {
			$offset = self::_getOffsetFilter()->filter($offset);
			
			$offset{0} = chr((ord($offset{0}) + 32)); //lcfirst()
		}
		
		return $offset;
	}
	
	/**
	 * Sets object state.
	 * 
	 * @param array Object members.
	 * @return Model_Base
	 */
	public function setOptions(array $options)
    {
        foreach ($options as $key => $value) {
			$this->$key = $value;
        }
		
        return $this;
    }
	
	/**
	 * Sets data mapper.
	 * 
	 * @param mixed Data mapper.
	 * @return Model_Base
	 */
    public function setMapper($mapper)
    {
        $this->mapper = $mapper;
		
        return $this;
    }

    /**
	 * Gets data mapper.
	 * 
	 * @return object
	 */
    public function getMapper()
    {
        if (null === $this->mapper) {
			$className = get_class($this);
			$className = substr_replace($className, '_Mapper_', strrpos($className, '_'), 1);
			
			if (@ class_exists($className)) { //($mapper = new $className()) instanceof Model_Mapper_Base
				$this->setMapper(new $className());
			}
			else {
				throw new Exception('There should be appropriate data mapper for ' . get_class($this) . ' model, called ' . $className . ' and it should be instance of Model_Mapper_Base class.');
			}
        }
		
        return $this->mapper;
    }
	
	//ArrayAccess interface implementation
	
	/**
	 * Required by the ArrayAccess implementation.
	 * Gets object member for the given offset.
	 *
	 * @param string Offset.
	 * @return mixed
	 */
	public function offsetGet($offset) 
	{
        return $this->$offset;
    }
	
	/**
	 * Required by the ArrayAccess implementation.
	 * Does nothing.
	 *
	 * @param string Offset.
	 * @param mixed Value to set.
	 * @return Model_Base
	 */
	public function offsetSet($offset, $value) 
	{
		$this->$offset = $value;
    }
	
	/**
	 * Required by the ArrayAccess implementation.
	 * Checks if offset exists.
	 *
	 * @param string Offset.
	 * @return bool
	 */
    public function offsetExists($offset) 
	{
        return (null === $this->$offset);
    }
	
	/**
	 * Required by the ArrayAccess implementation.
	 * Does nothing.
	 *
	 * @param string Offset.
	 * @return void
	 */
    public function offsetUnset($offset) 
	{
    }
	
	/**
	 * Fetches all rows.
	 * 
	 * @param string|array|Zend_Db_Table_Select An SQL WHERE clause or Zend_Db_Table_Select object. (optional)
     * @param string|array OPTIONAL An SQL ORDER clause. (optional)
     * @param int OPTIONAL An SQL LIMIT count. (optional)
     * @param int OPTIONAL An SQL LIMIT offset. (optional)
	 * @return array
	 */
	public function fetchAll($where = null, $order = null, $count = null, $offset = null)
    {
		return $this->getMapper()->fetchAll($where, $order, $count, $offset);
    }
	
	/**
	 * Fetches one row.
	 * 
	 * @param string|array|Zend_Db_Table_Select An SQL WHERE clause or Zend_Db_Table_Select object. (optional)
     * @param string|array An SQL ORDER clause. (optional)
	 * @return Model_Base
	 */
	public function fetchRow($where = null, $order = null)
    {
		return $this->getMapper()->fetchRow($where, $order);
    }
	
	/**
	 * Saves current record.
	 * 
	 * @return mixed
	 */
	public function save()
    {
		return $this->getMapper()->save($this);
    }
	
	/**
     * Converts this object to array.
     * 
     * @return array
     */
    public function toArray()
    {
		$retval = array();
		
		foreach ($this as $key=>$value) {
			if ($key != 'mapper') {
				$retval[$key] = $this->__get($key);
			}
		}
		
		return $retval;
    }
	
	/**
     * Provides access to some table gateway methods, directly from 
	 * model instance.
     *
     * @param string Method name.
     * @param array $args The parameters for the method.
     * @return mixed
     */
	public function __call($method, $args)
	{
		return call_user_func_array(array($this->getMapper(), $method), $args);
	}
}