<?php
/**
 * Webfolio CMS
 *
 * Copyright (C) 2009 Nikola Posa (http://www.nikolaposa.in.rs)
 *
 * This file is part of Webfolio CMS.
 *
 * Webfolio CMS is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 3 of the License, or 
 * (at your option) any later version.
 *
 * Webfolio CMS is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with Webfolio CMS. If not, see <http://www.gnu.org/licenses/>.
 */
 
/**
 * Hook that holds registered content plugins.
 *
 * @author Nikola Posa <posa.nikola@gmail.com>
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License
 */
class Model_ContentPlugins extends WebfolioCMS_HookContainer_HelperBased_Abstract
{
	const PRE_CONTENT = 'preContent';
	const POST_CONTENT = 'postContent';
	
	/**
	 * Content plugins are grouped by their type, so we're supplying 
	 * $groupingIndex to be used for grouping content plugins array.
	 *
	 * @var string
	 */
    protected $groupingIndex = 'type';
	
	/**
	 * Singleton instance.
	 *
	 * @var Model_ContentPlugins
	 */
    protected static $_instance = null;
	
	/**
	 * Defined by WebfolioCMS_HookContainer_Abstract.
	 *
	 * @return Model_ContentPlugins
	 */
	protected function __construct()
	{
		//Initializing stack.
		$this->data[self::PRE_CONTENT] = array();
		$this->data[self::POST_CONTENT] = array();
	}
	
	/**
	 * Defined by WebfolioCMS_Singleton_Interface.
     *
	 * Singleton pattern implementation.
	 *
	 * @return Model_ContentPlugins
	 */
	public static function getInstance()
	{
		if (null === self::$_instance) {
            self::$_instance = new self();
        }

        return self::$_instance;
	}
	
	/**
	 * Defined by WebfolioCMS_HookContainer_Interface. 
	 * 
	 * Retrieves registered content plugins.
	 *
	 * @param string|null Plugin type.
	 * @return array
	 */
	public static function get($type = null)
	{
		$instance = self::getInstance();
		
		$plugins = $instance->data;
		
		if ($type != null && array_key_exists($type, $plugins)) {
			return $plugins[$type];
		}
		else {
			return $plugins;
		}
	}
	
	/**
	 * Defined by WebfolioCMS_HookContainer_Interface. 
	 *
	 * Adds content plugin onto stack.
	 *
	 * Supported params for $contentPlugin are:
	 *	 type - Plugin type,
     *   helperName - Name of the view helper that will be used for rendering (string),
     *   userFriendlyName - Text that will be displayed in grid for managing content plugins in admin panel (string) - optional,
	 *   ctOmit - On which content types plugin will NOT have option to be rendered (array) - optional,
	 *   ctPick - On which content types plugin will have option to be rendered (array) - optional.
	 *
	 * @param array Content plugin data.
	 * @return void
	 */
	public static function add($contentPlugin)
	{
		$instance = self::getInstance();
		
		$instance->_doAdd($contentPlugin);
	}
	
	/**
	 * Defined by WebfolioCMS_HookContainer_HelperBased_Abstract.
	 *
	 * @param array Hook data.
	 * @return bool
	 */
	protected function isValid($plugin)
	{
		if (parent::isValid($plugin) && $this->checkPluginType($plugin[$this->groupingIndex])) {
			return true;
		}
		else {
			return false;
		}
	}
	
	/**
	 * Checks whether valid plugin type is supplied.
	 *
	 * @param string Plugin type.
	 * @return bool
	 */
	protected function checkPluginType($type)
	{
		if ($type != self::PRE_CONTENT && $type != self::POST_CONTENT) {
			return false;
		}
        
		return true;
	}
	
	/**
	 * Defined by WebfolioCMS_HookContainer_Abstract.
	 *
	 * Prepares data of some content plugin, before adding it into 
	 * plugins stack.
	 *
	 * @param array Content plugin data.
	 * @param array On which content types plugin will NOT have option to be rendered.
	 * @param array On which content types plugin will have option to be rendered.
	 * @return array
	 */
	protected function prepare($plugin)
	{
        $plugin['settings'] = array(
            'selections'=>array(),
            'topSeparator'=>0,
            'float'=>'none'
        );
		
		if (array_key_exists('ctOmit', $plugin)) {
			$ctOmit = (!is_array($plugin['ctOmit'])) ? array($plugin['ctOmit']) : $plugin['ctOmit'];
			$plugin['settings']['ctOmit'] = $ctOmit;
		}
		elseif (array_key_exists('ctPick', $plugin)) {
			$ctPick = (!is_array($plugin['ctPick'])) ? array($plugin['ctPick']) : $plugin['ctPick'];
			$plugin['settings']['ctPick'] = $ctPick;
		}
		
		unset($plugin['ctOmit']);
		unset($plugin['ctPick']);
		
		if (!array_key_exists('userFriendlyName', $plugin)) {
			$plugin['userFriendlyName'] = $this->getUserFriendlyName($plugin['helperName']);
		}
		
		return $plugin;
	}
	
	/**
	 * Generates user friendly name of some content plugin ViewHelper 
	 * name.
	 *
	 * @param string ViewHelper name.
	 * @return string
	 */
	protected function getUserFriendlyName($pluginFileName)
	{
		$pluginFileName = preg_replace('/wf/i', 'Webfolio', $pluginFileName);
		
		$cc2s = new Zend_Filter_Word_CamelCaseToSeparator();
		
		return $cc2s->filter($pluginFileName);
	}
}
?>