<?php
/**
 * Webfolio CMS
 *
 * Copyright (C) 2009 Nikola Posa (http://www.nikolaposa.in.rs)
 *
 * This file is part of Webfolio CMS.
 *
 * Webfolio CMS is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 3 of the License, or 
 * (at your option) any later version.
 *
 * Webfolio CMS is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with Webfolio CMS. If not, see <http://www.gnu.org/licenses/>.
 */
 
/**
 * WebfolioEntries model.
 *
 * Utilizes the Data Mapper pattern to persist data. Represents 
 * a single webfolio entry.
 * 
 * @author Nikola Posa <posa.nikola@gmail.com>
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License
 */
class Model_WebfolioEntries extends Model_Base
{
	/**
	 * Type of an entry (i.e. item).
	 *
	 * @var string
	 */
	protected $type;
	
	/**
	 * Id of a entry.
	 *
	 * @var int
	 */
	protected $id;

    /**
	 * Id of a user who submitted entry.
	 *
	 * @var int
	 */
	protected $userId;
	
	/**
	 * This can be either array with id of a parent entry and 
	 * postition in tree based on it, or only id of a parent.
	 *
	 * @var mixed
	 */
	protected $parentEntry;
	
	/**
	 * Entry's title.
	 *
	 * @var string
	 */
	protected $title;
	
	/**
	 * Entry's text.
	 *
	 * @var string
	 */
	protected $text;
	
	/**
	 * Datetime when entry was posted.
	 *
	 * @var string
	 */
	protected $datetime;
	
	/**
	 * Entry's title in url format.
	 *
	 * @var string
	 */
	protected $titleUrlFormat;

	/**
	 * Flag which tells whether to use comments in this webfolio 
	 * entry.
	 * 
	 * @var int
	 */
	protected $useComments;
	
	/**
	 * Status of a entry (published, ready, draft).
	 *
	 * @var string
	 */
	protected $status;
	
	/**
	 * Tags which are used for marking wf_entries-related caches. 
	 * Cleaning of cache is performed on every save() method call, 
	 * and caching entries marked with any of tags in $cachingTags 
	 * array are automatically cleaned from default cache 
	 * (Core/File).
	 *
	 * @var array
	 */
	protected $cachingTags = array('wf_entry');
	
	/**
	 * Constructor.
	 *
	 * @param string Type of an entry.
	 * @param array Data of some entry.
	 * @return Model_WebfolioEntries
	 */
	public function __construct($type, array $options = null)
    {
		$this->type = (string)$type;
		
		$this->cachingTags[] = 'wf_' . $this->type;
		
		$this->mapper = new Model_Mapper_WebfolioEntries($this->type);
		
        parent::__construct($options);
    }
	
	/**
	 * Retrieves entry type.
	 * 
	 * @return null|string
	 */
    public function getType()
    {
        return $this->type;
    }
	
	/**
	 * Sets id.
	 * 
	 * @param int Id of a entry.
	 * @return Model_WebfolioEntries
	 */
    public function setId($id)
    {
        $this->id = (int) $id;
		
        return $this;
    }

    /**
	 * Retrieves id.
	 * 
	 * @return null|int
	 */
    public function getId()
    {
        return $this->id;
    }

    /**
	 * Sets user_id.
	 *
	 * @param int Id of a user.
	 * @return Model_WebfolioEntries
	 */
    public function setUserId($userId)
    {
        $this->userId = (int) $userId;

        return $this;
    }

    /**
	 * Retrieves user_id.
	 *
	 * @return null|int
	 */
    public function getUserId()
    {
        return $this->userId;
    }
	
	/**
	 * Sets parent entry.
	 * 
	 * @param mixed Parent entry.
	 * @return Model_WebfolioEntries
	 */
    public function setParentEntry($parentEntry)
    {
        $this->parentEntry = $parentEntry;
		
        return $this;
    }

    /**
	 * Retrieves parent entry.
	 * 
	 * @return mixed
	 */
    public function getParentEntry()
    {
        return $this->parentEntry;
    }
	
	/**
	 * Sets title.
	 * 
	 * @param string Title of a webfolio entry.
	 * @return Model_WebfolioEntries
	 */
    public function setTitle($title)
    {
        $this->title = $title;
		
        return $this;
    }

    /**
	 * Retrieves title.
	 * 
	 * @return null|string
	 */
    public function getTitle()
    {
        return $this->title;
    }
	
	/**
	 * Sets text.
	 * 
	 * @param string Text of a webfolio entry.
	 * @return Model_WebfolioEntries
	 */
    public function setText($text)
    {
        $this->text = $text;
		
        return $this;
    }

    /**
	 * Retrieves text.
	 * 
	 * @return null|string
	 */
    public function getText()
    {
        return $this->text;
    }
	
	/**
	 * Retrieves shorter version of entry's text.
	 * 
	 * @return null|string
	 */
    public function getTeaser($length = 100, $delimiter = '.', $suffix = '...')
    {
		if ($this->text != null) {
			$teaserFilter = new WebfolioCMS_Filter_Teaser(array('length'=>$length, 'delimiter'=>$delimiter, 'suffix'=>$suffix));
			
			return $teaserFilter->filter($this->text);
		}
		
        return null;
    }
	
	/**
	 * Sets datetime.
	 * 
	 * @param string Datetime when webfolio entry was posted.
	 * @return Model_WebfolioEntries
	 */
    public function setDatetime($datetime)
    {
        $this->datetime = $datetime;
		
        return $this;
    }

    /**
	 * Retrieves datetime.
	 * 
	 * @return null|string
	 */
    public function getDatetime()
    {
        return $this->datetime;
    }
	
	/**
	 * Sets titleUrlFormat.
	 * 
	 * @param string Title of some entry in url format.
	 * @return Model_WebfolioEntries
	 */
    public function setTitleUrlFormat($titleUrlFormat)
    {
        $this->titleUrlFormat = $titleUrlFormat;
		
        return $this;
    }

    /**
	 * Retrieves titleUrlFormat.
	 * 
	 * @return null|string
	 */
    public function getTitleUrlFormat()
    {
        return $this->titleUrlFormat;
    }
	
	/**
	 * Sets useComments flag.
	 * 
	 * @param int useComments flag.
	 * @return Model_WebfolioEntries
	 */
    public function setUseComments($useComments)
    {
        $this->useComments = (int)$useComments;
		
        return $this;
    }

    /**
	 * Retrieves useComments flag.
	 * 
	 * @return null|int
	 */
    public function getUseComments()
    {
        return $this->useComments;
    }

	/**
	 * Sets status of a entry.
	 * 
	 * @param string Status of a entry.
	 * @return Model_WebfolioEntries
	 */
    public function setStatus($status)
    {
        $this->status = $status;
		
        return $this;
    }

    /**
	 * Retrieves status of a entry.
	 * 
	 * @return null|string
	 */
    public function getStatus()
    {
        return $this->status;
    }

	/**
	 * Saves current entry.
	 * 
	 * @return mixed
	 */
	public function save()
    {
		$this->generateTitleUrlFormat();
		
		if (!isset($this->datetime)) { //If we're saving entry for the first time, we should also set datetime.
			$this->datetime = new Zend_DB_Expr('NOW()'); 
		}

		//Cleaning default cache.
		Zend_Registry::get('CacheManager')->getCache('default')->clean(Zend_Cache::CLEANING_MODE_MATCHING_ANY_TAG, $this->cachingTags);
		
		return $this->getMapper()->save($this);
    }
	
	/**
	 * Generates title url format if needed.
	 * 
	 * @return void
	 */
	protected function generateTitleUrlFormat()
	{
		if (!$this->getId() || $this->getMapper()->getTitleById($this->getId()) != $this->getTitle()) { //New mode or title is changed?
			$addWhere = ($id = $this->getId()) ? array(WebfolioCMS_Filter_TitleUrlFormat::TABLE_ALIAS . '.id != ?'=>$id) : null;
			$tufFilter = new WebfolioCMS_Filter_TitleUrlFormat('wf_entries', 'title_url_format', $addWhere);
			$titleUrlFormat = $tufFilter->filter($this->getTitle());
			
			$this->setTitleUrlFormat($titleUrlFormat);
		}
	}
}
?>