<?php
/**
 * Webfolio CMS
 *
 * Copyright (C) 2009 Nikola Posa (http://www.nikolaposa.in.rs)
 *
 * This file is part of Webfolio CMS.
 *
 * Webfolio CMS is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 3 of the License, or 
 * (at your option) any later version.
 *
 * Webfolio CMS is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with Webfolio CMS. If not, see <http://www.gnu.org/licenses/>.
 */
 
/**
 * Helper model for getting Webfolio CMS settings.
 *
 * @author Nikola Posa <posa.nikola@gmail.com>
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License
 */
class Model_WebfolioSettings
{
	const NAME_KEY = 'name';
	const VALUE_KEY = 'value';
	const AUTOLOAD_KEY = 'autoload';
	
	/**
	 * Singleton instance.
	 *
	 * @var Model_WebfolioSettings
	 */
    protected static $_instance = null;
	
	/**
	 * Instance of WebfolioSettings model.
	 *
	 * @var WebfolioSettings
	 */
	protected $webfolioSettings;
	
	/**
	 * Array with all app settings, that is used as some kind of 
	 * cache, for quicker access.
	 *
	 * @var array
	 */
	protected $settings;
	
	/**
	 * Constructor.
	 *
	 * @return Model_WebfolioSettings
	 */
	protected function __construct()
	{
		$this->webfolioSettings = new Model_DbTable_WebfolioSettings();
		
		//Storing all settings in cache array.
		$this->settings = $this->_getAllSettings();
	}
	
	/**
	 * Returns an instance of Model_WebfolioSettings.
     *
	 * Singleton pattern implementation.
	 *
	 * @return Model_WebfolioSettings
	 */
    public static function getInstance()
    {
        if (null === self::$_instance) {
            self::$_instance = new self();
        }

        return self::$_instance;
    }
	
	/**
	 * Gets custom app setting.
	 *
	 * @param mixed Name(s) of the setting.
	 * @return mixed|null	  
	 */
	public static function getSettings($name = null)
	{
		$instance = self::getInstance();
		
		if ($name == null) {
			return $instance->settings;
		}
		else {
			if (is_array($name)) { //Array?
				if (count($name) == 1) {
					return $instance->_getSetting(current($name));
				}
				
				$result = array();
				foreach ($name as $n) {
					$result[$n] = $instance->_getSetting($n);
				}
				
				return $result;
			}
			elseif (is_string($name)) { //String?
				return $instance->_getSetting($name);
			}
			else { //Otherwise, return null.
				return null;
			}
		}
	}
	
	/**
	 * Returns $settings array.
	 *
	 * @param bool Whether to get all settings, or only those which are marked to be autoloaded.
	 * @return array
	 */
	public static function getAllSettings($all = false)
	{
		$instance = self::getInstance();
		
		if ($all == true) {
			return $instance->_getAllSettings(true);
		}
		else {
			return $instance->settings;
		}
	}
	
	/**
	 * Fetches one app setting.
	 *
	 * @return string|null
	 */
	protected function _getSetting($name)
	{
		if (!array_key_exists($name, $this->settings)) { //Setting isn't stored in cache? Retrieve setting from database.
			$result = $this->webfolioSettings->fetchRow($this->webfolioSettings->getAdapter()->quoteInto('name = ?', $name));
		
			if (!empty($result)) { //Setting exists?
				//Store setting in cache, for later use.
				$this->settings[$name] = $this->_unpackValue($result->__get('value'));
				
				return $this->settings[$name];
			}
			else {
				return null;
			}
		}
		else { //Setting is in cache? Simply get that setting.
			return $this->settings[$name];
		}
	}
	
	/**
	 * Fetches all app settings. This function is called only once, 
	 * when this class in instantiated, with purpose to store all 
	 * settings in cache.
	 *
	 * @param bool Whether to get all settings, or only those which are marked to be autoloaded.
	 * @return array
	 */
	protected function _getAllSettings($all = false)
	{
		$where = ($all == true) ? null : 'autoload = 1';
		
		$settings = $this->webfolioSettings->fetchAll($where)->toArray();
		
		$result = array();
		foreach($settings as $s) {
			$result[$s['name']] = $this->_unpackValue($s['value']);
		}
		
		return $result;
	}
	
	/**
	 * Add new webfolio setting.
	 *
	 * @param string Name of the setting.
	 * @param mixed Value of the setting.
	 * @param int Whether to autoload this setting on every request.
	 * @return void
	 */
	public static function addSetting($name, $value, $autoload = 0)
	{
		$instance = self::getInstance();
		
		if (null === $instance->_getSetting($name)) { //Only if setting does not already exists.
		
			$val = $instance->_prepareValue($value);
			$autoload = (int)$autoload;
			
			if ($autoload !== 1) { 
				$autoload = 0; 
			}
			
			$instance->webfolioSettings->insert(array(self::NAME_KEY=>$name, self::VALUE_KEY=>$val, self::AUTOLOAD_KEY=>$autoload));
			
			if ($autoload == 1) {
				$instance->settings[$name] = $val;
			}
		}
	}
	
	/**
	 * Add many webfolio setting at once.
	 *
	 * @param array Settings array.
	 * @return void
	 */
	public static function addSettings(array $settings)
	{
		if (array_key_exists(self::NAME_KEY, $settings) && array_key_exists(self::VALUE_KEY, $settings)) { //Only one setting is supplied?
			$autoload = (array_key_exists(self::AUTOLOAD_KEY, $settings)) ? $setting[self::AUTOLOAD_KEY] : 0; 
			
            self::addSetting($settings[self::NAME_KEY], $settings[self::VALUE_KEY], $autoload);
        } 
        else {
			foreach ($settings as $setting) {
				if (is_array($setting) && array_key_exists(self::NAME_KEY, $settings) && array_key_exists(self::VALUE_KEY, $settings)) {
					if (array_key_exists(self::NAME_KEY, $settings) && array_key_exists(self::VALUE_KEY, $settings)) {
						$autoload = (array_key_exists(self::AUTOLOAD_KEY, $settings)) ? $setting[self::AUTOLOAD_KEY] : 0;
						
						self::addSetting($setting[self::NAME_KEY], $setting[self::VALUE_KEY], $autoload);
					}
				} 
			}
		}
	}
	
	/**
	 * Updates value of some setting in database.
	 *
	 * @param string Name of the setting.
	 * @param mixed Value of the setting.
	 * @return int
	 */
	public static function updateSetting($name, $val)
	{
		$instance = self::getInstance();
		
		$value = $instance->_prepareValue($val);
			
		$where = $instance->webfolioSettings->getAdapter()->quoteInto('name = ?', $name);
		
		return $instance->webfolioSettings->update(array('value'=>$value), $where);
	}
	
	/**
	 * Deletes webfolio setting(s).
	 *
	 * @param mixed Single name of the setting or array with names of settings.
	 * @return int
	 */
	public static function deleteSettings($settings)
	{
		if (!is_array($settings)) {
			$settings = array($settings);
		}
		
		$rowsAffected = 0;
		
		$instance = self::getInstance();
		
		foreach ($settings as $name) {
			$where = $instance->webfolioSettings->getAdapter()->quoteInto('name = ?', $name);
			
			$rowsAffected += $instance->webfolioSettings->delete($where);
		}
		
		return $rowsAffected;
	}
	
	/**
	 * Prepares some value, before storing it into database.
	 *
	 * @param mixed Value of some setting.
	 * @return string
	 */
	protected function _prepareValue($val)
	{
		$value = null;
		
		if (is_array($val)) { //Array? Convert it to JSON.
			$value = Zend_Json_Encoder::encode($val);
		}
		elseif (!is_string($val)) {
			$value = (string)$val;
		}
		else {
			$value = $val;
		}
		
		return $value;
	}
	
	/**
	 * Checks whether some settings value from database might be 
	 * type of JSON.
	 *
	 * @param string Value of some setting.
	 * @return bool
	 */
	protected function isJson($value)
	{
		if (strlen($value) > 0 && (($value{0} == '{' || $value{0} == '[') && ($value{strlen($value)-1} == '}' || $value{strlen($value)-1} == ']'))) {
			return true;
		}
		else {
			return false;
		}
	}
	
	/**
	 * Converts value of some setting from its database form, into 
	 * valid PHP type (array, bool, string).
	 *
	 * @param mixed Database value of some setting.
	 * @return mixed
	 */
	protected function _unpackValue($value)
	{
		if ($this->isJson($value)) { //JSON?
			return Zend_Json_Decoder::decode($value);
		}
		else {
			return $value;
		}
	}
}
?>