<?php
/**
 * Webfolio CMS
 *
 * Copyright (C) 2009 Nikola Posa (http://www.nikolaposa.in.rs)
 *
 * This file is part of Webfolio CMS.
 *
 * Webfolio CMS is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 3 of the License, or 
 * (at your option) any later version.
 *
 * Webfolio CMS is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with Webfolio CMS. If not, see <http://www.gnu.org/licenses/>.
 */
 
/**
 * Base data mapper that should be extended by all Webfolio 
 * CMS ORM mappers.
 *
 * Implements the Data Mapper design pattern.
 * 
 * @author Nikola Posa <posa.nikola@gmail.com>
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License
 */
class Model_Mapper_Base
{
	/**
	 * Table gateway.
	 *
     * @var Zend_Db_Table_Abstract
     */
	protected $dbTable;
	
	/**
	 * Specifies Zend_Db_Table instance to use for data operations.
	 * 
	 * @param string|Zend_Db_Table_Abstract Zend_Db_Table instance.
	 * @return Model_Mapper_Base
	 */
	public function setDbTable($dbTable)
    {
        if (is_string($dbTable)) {
            $dbTable = new $dbTable();
        }
        if (!$dbTable instanceof Zend_Db_Table_Abstract) {
            throw new Exception('Invalid table data gateway provided.');
        }
		
        $this->dbTable = $dbTable;
		
        return $this;
    }
	
	/**
	 * Gets registered Zend_Db_Table instance.
	 * 
	 * @return Zend_Db_Table_Abstract
	 */
	public function getDbTable()
    {
        if (null === $this->dbTable) {
			$className = str_replace('Mapper', 'DbTable', get_class($this));
			
			if (@ class_exists($className)) {
				$this->setDbTable($className);
			}
			else {
				throw new Exception('There should be table gateway in file name: ' . $className . '.');
			}
        }
		
        return $this->dbTable;
    }
	
	/**
	 * Saves a comment entry.
	 * 
	 * @param Model_Base Model instance.
	 * @return mixed
	 */
	public function save(Model_Base $model)
	{
		$data = $this->prepareData($model);
		
		$primary = $this->getDbTable()->info(Zend_Db_Table_Abstract::PRIMARY);
		$primary = $primary[1];
		
		if (null == ($id = $model->$primary)) { //Saving for the first time?
			$retval = $this->getDbTable()->insert($data);
			
			$model->$primary = $retval;
			
			return $retval;
        } 
		else {
			$where = $this->getDbTable()->getAdapter()->quoteInto($primary . ' = ?', $id, Zend_Db::INT_TYPE);
			
            return $this->getDbTable()->update($data, $where);
        }
	}
	
	/**
	 * Prepares and packs data from the model that will be saved to 
	 * database.
	 * 
	 * @param Model_Base Model instance.
	 * @return array
	 */
	protected function prepareData($model)
	{
		$data = array();
		
		$columns = array_keys($this->getDbTable()->info(Zend_Db_Table_Abstract::METADATA));
		
		$primary = $this->getDbTable()->info(Zend_Db_Table_Abstract::PRIMARY);
		if (count($primary) > 1) { //Compound key?
			throw new Exception('Tables with compound primary key are not currently supported.');
		}
		
		$primary = $primary[1];
		
		foreach ($columns as $column) {
			if ($column != $primary) { 
				$data[$column] = $model[$column];
			}
		}
		
		return $data;
	}
	
	/**
	 * Fetches all rows.
	 * 
	 * @param string|array|Zend_Db_Table_Select An SQL WHERE clause or Zend_Db_Table_Select object. (optional)
     * @param string|array OPTIONAL An SQL ORDER clause. (optional)
     * @param int OPTIONAL An SQL LIMIT count. (optional)
     * @param int OPTIONAL An SQL LIMIT offset. (optional)
	 * @return array
	 */
	public function fetchAll($where = null, $order = null, $count = null, $offset = null)
    {
        $result = $this->getDbTable()->fetchAll($where, $order, $count, $offset);
		
        $models = array();
		
        foreach ($result as $row) {
			$models[] = $this->populateModel($row);
        }
		
        return $models;
    }
	
	/**
	 * Fetches one row.
	 * 
	 * @param string|array|Zend_Db_Table_Select An SQL WHERE clause or Zend_Db_Table_Select object. (optional)
     * @param string|array OPTIONAL An SQL ORDER clause. (optional)
	 * @return Model_Base|null
	 */
	public function fetchRow($where = null, $order = null)
    {
        $result = $this->getDbTable()->fetchRow($where, $order);
		
		if ($result == null) {
			return null;
		}

        return $this->populateModel($result);
    }
	
	/**
	 * Creates model instance and populate its properties with 
	 * supplied rowset of data.
	 * 
	 * @param array Result row from the database.
	 * @return Model_Base
	 */
	protected function populateModel($row)
	{
		if (method_exists($row, 'toArray')) {
			$row = $row->toArray();
		}
		
		$modelClassName = $this->getModelClassName();
		
		$model = new $modelClassName();
		
		foreach ($row as $key=>$value) {
			$model[$key] = $value;
		}
		
		return $model;
	}
	
	/**
	 * Generates model class name, based on models/mappers 
	 * resources namespace rules.
	 * 
	 * @return string
	 */
	protected function getModelClassName()
	{
		return str_replace('Mapper_', '', get_class($this));
	}
	
	/**
     * Provides access to some table gateway methods.
     *
     * @param string Method name.
     * @param array $args The parameters for the method.
     * @return mixed
     */
	public function __call($method, $args)
	{
		if (!method_exists($this->getDbTable(), $method)) {
			throw new Exception('Method ' . $method . ' does not exist in ' . get_class($this->getDbTable()) . ' table gateway class.');
		}
		
		$result = call_user_func_array(array($this->getDbTable(), $method), $args);
		
		$retval = $result;
		
		if ($result) {
			if ($result instanceof Zend_Db_Table_Rowset) {
				$models = array();
				
				foreach ($result as $row) {
					$models[] = $this->populateModel($row);
				}
				
				$retval = $models;
			}
			elseif ($result instanceof Zend_Db_Table_Row) {
				$retval = $this->populateModel($result);
			}
		}
		
		return $retval;
	}
}
?>