<?php
/**
 * Webfolio CMS
 *
 * Copyright (C) 2009 Nikola Posa (http://www.nikolaposa.in.rs)
 *
 * This file is part of Webfolio CMS.
 *
 * Webfolio CMS is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 3 of the License, or 
 * (at your option) any later version.
 *
 * Webfolio CMS is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with Webfolio CMS. If not, see <http://www.gnu.org/licenses/>.
 */
 
/**
 * Data mapper for all WebfolioEntries instances.
 *
 * Implements the Data Mapper design pattern.
 * 
 * @author Nikola Posa <posa.nikola@gmail.com>
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License
 */
class Model_Mapper_WebfolioEntries extends Model_Mapper_Base
{
	const PARENT_ALIAS = 'parent_entry';
	
	/**
	 * Webfolio entry type (i.e. item).
	 * 
	 * @var string
	 */
	protected $type;
	
	/**
	 * Constructor.
	 *
	 * @param string Type of an entry.
	 * @return Model_Mapper_WebfolioEntries
	 */
	public function __construct($type)
    {
		$this->type = $type;
		
		$this->dbTable = new Model_DbTable_WebfolioEntries();
    }
	
	/**
	 * Saves a webfolio entry.
	 * 
	 * @param Model_WebfolioEntries WebfolioEntries model instance.
	 * @return mixed
	 */
	public function save(Model_WebfolioEntries $webfolioEntry)
	{
		$data = $this->prepareData($webfolioEntry);
		
		$objectiveId = null;
		$position = NP_Db_Table_NestedSet::LAST_CHILD;
		
		if ($objective = $webfolioEntry->getParentEntry()) {	
			$objectiveId = $objective['id'];
			$position = ($objective['position']) ? $objective['position'] : NP_Db_Table_NestedSet::LAST_CHILD;
		}
		
		$id = $webfolioEntry->getId();
        if (null != $id || $id != 0) {	
			if ($objectiveId !== null) {
				return $this->dbTable->updateNode($data, $id, $objectiveId, $position);
			}
			else {
				$where = $this->dbTable->getAdapter()->quoteInto('id = ?', $id, Zend_Db::INT_TYPE);
				return $this->dbTable->update($data, $where);
			}
        }
		else {
			$retval = $this->dbTable->insert($data, $objectiveId, $position);
			
			$webfolioEntry->id = $retval;
			
			return $retval;
		}
	}
	
	/**
	 * Defined by Model_BaseMapper.
	 * 
	 * @param string|array|Zend_Db_Table_Select An SQL WHERE clause or Zend_Db_Table_Select object. (optional)
     * @param string|array OPTIONAL An SQL ORDER clause. (optional)
     * @param int OPTIONAL An SQL LIMIT count. (optional)
     * @param int OPTIONAL An SQL LIMIT offset. (optional)
	 * @return array
	 */
	public function fetchAll($where, $order, $count, $offset)
	{
		$where = $this->getWhereClause($where);
		
		$result = $this->dbTable->fetchAll($where, false, self::PARENT_ALIAS, $order, $count, $offset);
		
        $webfolioEntries = array();
		
        foreach ($result as $row) {
			$webfolioEntries[] = $this->populateModel($row);
        }
		
        return $webfolioEntries;
	}
	
	/**
	 * Defined by Model_BaseMapper.
	 * 
	 * @param mixed An SQL WHERE clause or Zend_Db_Table_Select object.
	 * @param string|array OPTIONAL An SQL ORDER clause. (optional)
	 * @return Model_WebfolioEntries|null
	 */
	public function fetchRow($where, $order)
	{
		$where = $this->getWhereClause($where);
		
		$result = $this->dbTable->fetchRow($where, self::PARENT_ALIAS, $order);
		
		if ($result == null) {
			return null;
		}
		
        return $this->populateModel($result);
	}
	
	/**
	 * Defined by Model_BaseMapper.
	 * 
	 * @param array Result row from the database.
	 * @return Model_Base
	 */
	protected function populateModel($row)
	{
		if (method_exists($row, 'toArray')) {
			$row = $row->toArray();
		}
		
		$modelClassName = $this->getModelClassName();
		
		$model = new $modelClassName($row['type']);
		unset($row['type']);
		
		foreach ($row as $key=>$value) {
			$model[$key] = $value;
		}
		
		return $model;
	}
	
	/**
	 * Generates WHERE clauses array which will be used in fetchAll() 
	 * and fetchRow() methods.
	 * 
	 * @param mixed An SQL WHERE clause or Zend_Db_Table_Select object.
	 * @return array
	 */
	protected function getWhereClause($where)
	{
		//Setting entry type condition by default.
		$whereArray = array(Model_DbTable_WebfolioEntries::LEFT_TBL_ALIAS . '.type = ?' => $this->type);
		
		if ($where != null) {	
			if ($where instanceof Zend_Db_Table_Select) {
				$where = $where->__toString();
			}
			
			if (!is_array($where)) {
				$where = array($where);
			}
			
			$whereArray = array_merge($where, $whereArray);
		}
		
		return $whereArray;
	}
}
?>