<?php
/**
 * Webfolio CMS
 *
 * Copyright (C) 2009 Nikola Posa (http://www.nikolaposa.in.rs)
 *
 * This file is part of Webfolio CMS.
 *
 * Webfolio CMS is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 3 of the License, or 
 * (at your option) any later version.
 *
 * Webfolio CMS is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with Webfolio CMS. If not, see <http://www.gnu.org/licenses/>.
 */
 
/**
 * Controller for managing groups permissions.
 *
 * @author Nikola Posa <posa.nikola@gmail.com>
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License
 */
class Admin_GroupsPermissionsController extends Zend_Controller_Action
{
    /**
	 * Groups db table model instance.
	 *
	 * @var Model_DbTable_WebfolioGroups
	 */
	protected $_groupsDbTable;

    /**
	 * Groups permissions db table model instance.
	 *
	 * @var Model_DbTable_WebfolioGroupAcl
	 */
	protected $_groupAclDbTable;

    /**
	 * Groups permissions that should not be removed.
	 *
	 * @var array
	 */
    private $_readOnlyACLs = array(
        array('group_id'=>1, 'resource'=>'admin_groups-permissions', 'privilege'=>'index'),
        array('group_id'=>1, 'resource'=>'admin_groups-permissions', 'privilege'=>'edit')
    );

    /**
	 * Initialize object.
	 *
	 * @return void
	 */
	public function init()
    {
		$this->_groupsDbTable = new Model_DbTable_WebfolioGroups();

        $this->_groupAclDbTable = new Model_DbTable_WebfolioGroupAcl();
    }

	/**
	 * Displays form for editing groups permissions.
	 * 
	 * @return void
	 */
	public function indexAction()
	{
        if ($this->_request->isPost() && WebfolioCMS_Common::isUserAllowed(WebfolioCMS_Auth::getInstance()->getIdentity()->group_id, 'admin_groups-permissions', 'edit')) {
            $formData = $this->_request->getPost();
            
            $data = array();
            if (isset($formData['acl'])) {
                //Preparing data that will be inserted.
                foreach ($formData['acl'] as $groupId => $rules) {
                    foreach ($rules as $rule) {
                        $rule = Zend_Json_Decoder::decode(html_entity_decode($rule));
                        $dataRow = array('group_id'=>$groupId, 'resource'=>$rule['resource'], 'privilege'=>$rule['privilege']);
                        if (isset($rule['assert'])) {
                            $dataRow['assert'] = $rule['assert'];
                        }

                        $data[] = $dataRow;
                    }
                }
            }

            //If some of the read-only ACL is not supplied, we should add it.
            foreach ($this->_readOnlyACLs as $acl) {
                if (!in_array($acl, $data)) {
                    $data[] = $acl;
                }
            }
            
            //Prepending default (read-only) ACL rules to the supplied data.
            $data = array_merge(Model_DbTable_WebfolioGroupAcl::getDefaultGroupACLs(), $data);

            try {
                //Deleting all data.
                $sql = 'TRUNCATE TABLE ' . $this->_groupAclDbTable->info('name');
                $this->_groupAclDbTable->getAdapter()->query($sql);

                //Inserting new data.
                foreach ($data as $row) {
                    $this->_groupAclDbTable->insert($row);
                }

                //Clearing ACL cache.
                Zend_Registry::get('CacheManager')->getCache('default')->remove(WebfolioCMS_Acl::CACHE_ID);

                $this->view->message = 'Groups permission rules have been successfully saved.';
				$this->view->ok = true;
            }
            catch (Exception $ex) {
                $this->view->message = 'An error occured and groups permission rules can not be saved at the momemnt.';
				$this->view->ok = false;
            }
        }

        $this->view->groups = $this->_groupsDbTable->fetchAll(null, 'name ASC');
        $this->view->registeredGroupPermissions = Admin_Model_WFAdminGroupPermissions::get();
        $this->view->groupsPermissionsValues = $this->_getGroupsPermissionsScheme();
	}

    /**
	 * Retrieves groups permissions scheme stored in database, and
     * also groupes rules by group_id column.
	 *
	 * @return array
	 */
    protected function _getGroupsPermissionsScheme()
    {
        $groupsPermissions = array();
        $groupsPermissionsData = $this->_groupAclDbTable->fetchAll();
        foreach ($groupsPermissionsData as $row) {
            $row = $row->toArray();

            if ($row['group_id']) {
                unset($row['id']);

                $groupId = $row['group_id'];
                unset($row['group_id']);

                if (!$row['assert']) {
                    unset($row['assert']);
                }

                $groupsPermissions[$groupId][] = $row;
            }
        }

        return $groupsPermissions;
    }
}
?>