<?php
/**
 * Webfolio CMS
 *
 * Copyright (C) 2009 Nikola Posa (http://www.nikolaposa.in.rs)
 *
 * This file is part of Webfolio CMS.
 *
 * Webfolio CMS is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 3 of the License, or 
 * (at your option) any later version.
 *
 * Webfolio CMS is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with Webfolio CMS. If not, see <http://www.gnu.org/licenses/>.
 */
 
/**
 * Admin panel controller for handling requests for managing 
 * webfolio items.
 *
 * @author Nikola Posa <posa.nikola@gmail.com>
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License
 */
class Admin_ItemsController extends Zend_Controller_Action 
{		
	/**
	 * Webfolio items model instance.
	 * 
	 * @var Admin_Model_WebfolioEntries
	 */
	protected $webfolioItems;
	
	/**
	 * Webfolio items form instance.
	 * 
	 * @var Admin_Form_Item
	 */
	protected $itemForm;
	
	/**
	 * Initialize object.
	 * 
	 * @return void
	 */
	public function init()
    {
		$this->webfolioItems = new Admin_Model_WebfolioEntries('item');
		
		$this->itemForm = new Admin_Form_Item();
    }
	
	/**
	 * Default action. Displays webfolio items grid.
	 * 
	 * @return void
	 */
	public function indexAction()
	{	
	}
	
	/**
	 * Action for adding new webfolio item.
	 * 
	 * @return void
	 */
	public function newAction()
	{
		//We don't need datetime and status fields in new mode.
		$this->itemForm->removeElement('datetime');
		$this->itemForm->removeElement('status');
		$this->itemForm->removeElement('delete');
		
		if ($this->_request->isPost()) { //Form submitted?
			$formData = $this->_request->getPost();
				
			if ($this->itemForm->isValid($formData)) { //Valid data?	
				try {	
					$data = $this->itemForm->getValues();
					$data['status'] = 'ready';
					
					//Did user clicked on 'Publish'?
					if (isset($formData['publish'])) {
						$data['status'] = 'published';
					}
					
					$this->saveItem($data);
						
					if (isset($formData['save'])) {
						//Helper session entry that will just let us know that user has just saved his item, after we redirect to manage controller.
						$this->_helper->flashMessenger('Your webfolio item data has been saved.');
								
						$this->_helper->redirector->gotoRoute(array('item'=>$this->webfolioItems->getTitleUrlFormat()), 'admin_items_edit', true);
					}
					elseif (isset($formData['publish'])) {
						$this->_helper->redirector('index', 'items', 'admin');
					}
				}
				catch (Exception $ex) {
					$this->view->message = $ex->getMessage(); //'An error occured and your item can not be saved/published at the moment.';
					$this->view->ok = false;
				}
			}
			else {
				$this->view->message = 'You have validation error(s) in your form.';
				$this->view->ok = false;
			}
		}
		
		$this->view->form = $this->itemForm;
	}
	
	/**
	 * Action for editing webfolio item.
	 * 
	 * @return void
	 */
	public function editAction()
	{	
		if ($item = $this->_request->getParam('item')) { 
			$where = $this->webfolioItems->getMapper()->getDbTable()->getAdapter()->quoteInto('title_url_format = ?', $item);
			
			$webfolioItem = $this->webfolioItems->fetchRow($where);
			$webfolioItem['datetime'] = date('Y-m-d', strtotime($webfolioItem['datetime']));
			
			if ($this->_request->isPost()) { //Form submitted?
				$formData = $this->_request->getPost();
				
				if ($this->itemForm->isValid($formData)) { //Valid data?	
					$data = $this->itemForm->getValues();
					
					//Did user clicked on 'Save' or 'Publish'?
					if ($data['status'] == 'draft' && isset($formData['save'])) {
						$data['status'] = 'ready';
					}
					elseif (isset($formData['publish'])) {
						$data['status'] = 'published';
					}
					
					try {
						$this->saveItem($data, $webfolioItem);
						
						$this->itemForm->getElement('status')->setValue($webfolioItem->status);
						
						if (isset($formData['save'])) {	
							$this->_helper->flashMessenger('Your webfolio item data has been saved.');
						}
						elseif (isset($formData['publish'])) {
							$this->_helper->flashMessenger('Your webfolio item has been successfully published.');
						}
						
						$this->_helper->redirector->gotoRoute(array('item'=>$webfolioItem->getTitleUrlFormat()), 'admin_items_edit', true);
					}
					catch (Exception $ex) {
						$this->view->message = $ex->getMessage(); //'An error occured and your item can not be saved/published at the moment.';
						$this->view->ok = false;
					}
				}
				else {
					$this->view->message = 'You have validation error(s) in your form.';
					$this->view->ok = false;
				}
			}
			else { //Display form for editing item.
				if ($webfolioItem == null) {
					$this->view->error = 'Unexisting item.';
				}
				else {
					$this->itemForm->populate($webfolioItem->toArray());
					
					if ($this->_helper->flashMessenger->hasMessages()) {
						$this->view->message = current($this->_helper->flashMessenger->getMessages());
						$this->view->ok = true;
					}
				}
			}
			
			$this->view->form = $this->itemForm;
		}
	}
	
	/**
	 * Saves webfolio item data.
	 * 
	 * @param array Data from the item form.
	 * @param Admin_Model_WebfolioEntries WebfolioEntries instance that is already holding some data (optional).
	 * @return void
	 */
	protected function saveItem($data, $activeRecord = null)
	{
		$webfolioItem = ($activeRecord != null) ? $activeRecord : $this->webfolioItems;
		
		if ($data['treePosition'] || $data['parentEntry'] != $webfolioItem->getParentEntry()) { //We should update tree position only if user really decided to change it.
			$data['parentEntry'] = array('id'=>(int)$data['parentEntry'], 'position'=>$data['treePosition']);
		}
		else {
			$data['parentEntry'] = null;
		}
		
		//Modifying data in active record.
		$webfolioItem->setOptions($data);
		
		$dbHandler = Zend_Registry::get('dbHandler');
		$dbHandler->beginTransaction();
		
		try {
			//Saving submitted data.
			$webfolioItem->save();
			
			$dbHandler->commit();
		}
		catch (Zend_Db_Exception $ex) {
			$dbHandler->rollBack();
			
			throw $ex;
		}
	}
}
?>