<?php
/**
 * Webfolio CMS
 *
 * Copyright (C) 2009 Nikola Posa (http://www.nikolaposa.in.rs)
 *
 * This file is part of Webfolio CMS.
 *
 * Webfolio CMS is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 3 of the License, or 
 * (at your option) any later version.
 *
 * Webfolio CMS is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with Webfolio CMS. If not, see <http://www.gnu.org/licenses/>.
 */
 
/**
 * Form for editing user's profile.
 *
 * @author Nikola Posa <posa.nikola@gmail.com>
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License
 */
class Admin_Form_UserProfile extends Admin_Form_AbstractForm
{
    /**
     * Default display group class
     * @var string
     */
    protected $_defaultDisplayGroupClass = 'WebfolioCMS_Form_DisplayGroup';

    /**
     * Should we disable loading the default decorators?
     * @var bool
     */
    protected $_disableLoadDefaultDecorators = true;
    
	/**
	 * Initializing form.
	 *
	 * @return Zend_Form
	 */
	public function init()
	{
		$username = new Zend_Form_Element_Text('username', array('required'=>true, 'disableLoadDefaultDecorators'=>true));
		$username->addFilter('StringTrim');
        $regexValidator = new Zend_Validate_Regex('/^[a-z0-9\-_\.@\s]{3,}$/i');
        $regexValidator->setMessage('Username must be at least 3 characters long, and it should contain only alfanumeric characters, including these signs: \'-\', \'_\', \'.\' and \'@\'.');
		$username->addValidator($regexValidator);

        $password = new Zend_Form_Element_Password('password', array('required'=>false, 'disableLoadDefaultDecorators'=>true));
		$notEmpty = new Zend_Validate_NotEmpty();
		$notEmpty->setMessage('Please enter both password and re-password.');
		$strLength = new Zend_Validate_StringLength(6);
		$strLength->setMessage('Password must be at least 6 characters long.', 'stringLengthTooShort');
		$password->addValidators(array($notEmpty, $strLength));
        
        $userEmail = new Zend_Form_Element_Text('email', array('required'=>true, 'disableLoadDefaultDecorators'=>true));
		$userEmail->addFilter('StringTrim');
		$userEmail->addValidator(new Zend_Validate_EmailAddress());
		$userEmail->addErrorMessage('Please supply valid email address.');

        $group = new Zend_Form_Element_Select('groupId', array('required'=>true, 'registerInArrayValidator'=>false, 'disableLoadDefaultDecorators'=>true));
        $group->addErrorMessage('Please choose some group.');

        $publicName = new Zend_Form_Element_Select('publicName', array('required'=>true, 'registerInArrayValidator'=>false, 'disableLoadDefaultDecorators'=>true));

        $userId = new Zend_Form_Element_Hidden('id', array('disableLoadDefaultDecorators'=>true));

        $userDataSubForm = new Zend_Form_SubForm(array('disableLoadDefaultDecorators'=>true));
        $userDataSubForm->addElements(array($username, $password, $userEmail, $group, $publicName, $userId));
        
        $firstName = new Zend_Form_Element_Text('firstName', array('required'=>false, 'disableLoadDefaultDecorators'=>true));
		$firstName->addFilters(array('StringTrim', 'StripTags'));

        $lastName = clone $firstName;
        $lastName->setName('lastName');
        $lastName->setLabel('Last name');

        $website = new Zend_Form_Element_Text('website', array('required'=>false, 'disableLoadDefaultDecorators'=>true));
		$website->addFilter('StringTrim');
		$website->addValidator(new Zend_Validate_Callback(array('Zend_Uri', 'check')));
		$website->addErrorMessage('Please supply valid website address.');

        $userMetaSubForm = new Zend_Form_SubForm(array('disableLoadDefaultDecorators'=>true));
        $userMetaSubForm->addElements(array($firstName, $lastName, $website));

        $this->addSubForms(array('user'=>$userDataSubForm, 'user_meta'=>$userMetaSubForm));
        
		$rePassword = new Zend_Form_Element_Password('re_password', array('required'=>false, 'ignore'=>true, 'disableLoadDefaultDecorators'=>true));

        $this->addElement($rePassword);
	}

    /**
     * Defined by Zend_Form.
     *
     * Injecting custom pre-validation logic.
     *
     * @param array $data
     * @return bool
     */
	public function isValid($data)
	{
        if (($userName = $this->getSubForm('user')->getElement('username')) != null) {
            $dbHandler = Zend_Registry::get('dbHandler');
            $where = ($data['user']['id']) ? $dbHandler->quoteInto('id != ?', $data['user']['id'], Zend_Db::INT_TYPE) : null;
            $noRecordExists = new Zend_Validate_Db_NoRecordExists('wf_users', 'username', $where, $dbHandler);
            $noRecordExists->setMessage('That username already exists.');
            $userName->addValidator($noRecordExists);
        }
        
		if (strlen($data['user']['password']) > 0 || strlen($data['re_password']) > 0) {
			$this->user->getElement('password')->setRequired(true);
            $notEmpty = new Zend_Validate_NotEmpty();
            $notEmpty->setMessage('Please retype password.');
            $identical = new Zend_Validate_Identical($data['user']['password']);
            $identical->setMessage('Passwords in both fileds must match.');
			$this->getElement('re_password')->setRequired(true)->addValidators(array($notEmpty, $identical));
		}
		
		return parent::isValid($data);
	}
}
?>