<?php
/**
 * Webfolio CMS
 *
 * Copyright (C) 2009 Nikola Posa (http://www.nikolaposa.in.rs)
 *
 * This file is part of Webfolio CMS.
 *
 * Webfolio CMS is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 3 of the License, or 
 * (at your option) any later version.
 *
 * Webfolio CMS is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with Webfolio CMS. If not, see <http://www.gnu.org/licenses/>.
 */
 
/**
 * Form for publishing/editing some webfolio entry.
 *
 * @author Nikola Posa <posa.nikola@gmail.com>
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License
 */
abstract class Admin_Form_WFEntry extends Admin_Form_AbstractForm
{
	const REGISTRY_KEY = 'WFEntryForm';

    /**
     * Form metadata and attributes
     * @var array
     */
    protected $_attribs = array(
        'class' => 'wf_entry_form'
    );

    /**
     * Default display group class
     * @var string
     */
    protected $_defaultDisplayGroupClass = 'WebfolioCMS_Form_DisplayGroup';
    
	/**
     * Should we disable loading the default decorators?
	 *
     * @var bool
     */
    protected $_disableLoadDefaultDecorators = true;
	
	/**
     * Form type, retrieved from class name.
     *
     * @var string
     */
	protected $_type;
	
	/**
     * Helper array which stores names of added plugin elements.
     *
     * @var array
     */
	protected $_pluginsElements = array();
	
	/**
	 * Constructor.
	 *
	 * @param mixed $options
	 * @return void
	 */
	public function __construct($options = null)
	{
		$this->addPrefixPath('WebfolioCMS_Form_Decorator', BASE_PATH . '/libs/WebfolioCMS/Form/Decorator', 'decorator');
		
		$className = get_class($this);
		$this->_type = strtolower(substr($className, strrpos($className, '_')+1));
		
		$title = new Zend_Form_Element_Text('title', array('label'=>'Title:', 'required'=>true, 'size'=>40, 'disableLoadDefaultDecorators'=>true));
		$title->addFilters(array('StringTrim', new Zend_Filter_HtmlEntities(array('quotestyle'=>ENT_QUOTES, 'charset'=>'UTF-8', 'doublequote'=>false))));
		$notEmpty = new Zend_Validate_NotEmpty();
		$notEmpty->setMessage('Please supply valid title (max. 100 characters)');
		$title->addValidator($notEmpty);
		$title->addDecorator('Label')
			->addDecorator(new WebfolioCMS_Form_Decorator_Errors())
			->addDecorator('ViewHelper', array('separator'=>'<br />'))
			->addDecorator('HtmlTag', array('tag'=>'p'));
		$this->addElement($title);
		
		$text = new Zend_Form_Element_Textarea('text', array('label'=>'Text:', 'required'=>true, 'helper'=>'tinyMCEeditor', 'tmce_height'=>'300', 'disableLoadDefaultDecorators'=>true));
		$text->addFilter('StringTrim');
		$text->addErrorMessage('Please supply some text.');
		$text->addDecorator('Label')
			->addDecorator(new WebfolioCMS_Form_Decorator_Errors())
			->addDecorator('ViewHelper', array('separator'=>'<br />'))
			->addDecorator('HtmlTag', array('tag'=>'p'));
		$this->addElement($text);
		
		$treePosition = new Zend_Form_Element_Radio('treePosition', array('registerInArrayValidator'=>false, 'required'=>false, 'disableLoadDefaultDecorators'=>true));
		
		$parentEntry = new Zend_Form_Element_Select('parentEntry', array('registerInArrayValidator'=>false, 'required'=>false, 'disableLoadDefaultDecorators'=>true));
		$parentEntry->addErrorMessage('Please choose some parent item.');
		
		$this->addElements(array($treePosition, $parentEntry));
		
		$this->getView()->addScriptPath(APPLICATION_PATH . '/modules/admin/views/parts'); 
		$this->addDisplayGroup(
			array('treePosition', 'parentEntry'),
			'hierarchyPosition',
			array(
				'legend' =>'Hierarchy position', 
				'decorators' => array(new Zend_Form_Decorator_ViewScript(array('viewScript'=>'entry_hierarchy_position.phtml')))
			)
		);
		
		$useComments = new Zend_Form_Element_Checkbox('useComments', array('label'=>'Allow comments:', 'required'=>true, 'disableLoadDefaultDecorators'=>true));
		$useComments->addDecorator('Label')
			->addDecorator('ViewHelper')
			->addDecorator(new WebfolioCMS_Form_Decorator_Info(array('text'=>'Whether or not you want to enable posting of comments on this item\'s page.')))
			->addDecorator('HtmlTag', array('tag'=>'p'));
		$this->addElement($useComments);
		
		$this->addDisplayGroup(
			array('useComments'),
			'comments',
			array( 
				'legend' =>'Comments settings'
			)
		);
		
		$this->initWFEntryFormPlugins();
		
		$id = new Zend_Form_Element_Hidden('id', array('disableLoadDefaultDecorators'=>true));
		$id->addDecorator('ViewHelper');
		$this->addElement($id);
		
		$titleUrlFormat = new Zend_Form_Element_Hidden('titleUrlFormat', array('disableLoadDefaultDecorators'=>true));
		$titleUrlFormat->addDecorator('ViewHelper');
		$this->addElement($titleUrlFormat);
		
		$this->getView()->headLink()->appendStylesheet($this->getView()->baseUrl('libs/jQuery/jquery-ui-datepicker/css/flick/jquery-ui-datepicker.css'));
		$this->getView()->headScript(
			Zend_View_Helper_HeadScript::FILE, 
			$this->getView()->baseUrl('libs/jQuery/jquery-ui-datepicker/js/jquery-ui-datepicker.min.js')
		);
		
		$date = new ZendX_JQuery_Form_Element_DatePicker(
			'datetime', 
			array(
				'label'=>'Date/time:', 
				'required'=>true, 
				'disableLoadDefaultDecorators'=>true, 
				'jQueryParams'=>array(
					'dateFormat'=>'yy-mm-dd',
					'changeMonth'=>true,
					'changeYear'=>true,
					'appendText'=>' (YYYY-MM-DD HH:mm:ss)',
					'onSelect'=> new Zend_Json_Expr('function(dateText, inst) { $(this).val(dateText + \' 00:00:00\'); }')
				)
			)
		);
		$date->addValidator(new Zend_Validate_Date('YYYY-MM-dd HH:mm:ss'));
		$date->addErrorMessage('Please supply valid date/time.');
		$date->addDecorator('Label')
				 ->addDecorator(new WebfolioCMS_Form_Decorator_Errors())
				 ->addDecorator(new WebfolioCMS_Form_Decorator_Info(array('text'=>'This date/time format will not affect date and time format that\'t you\'ve set in form for managing general settings.')))
				 ->addDecorator('UiWidgetElement', array('separator'=>'<br />'))
				 ->addDecorator('HtmlTag', array('tag'=>'p'));
		$this->addElement($date);
		
		$status = new Zend_Form_Element_Select('status', array('label'=>'Publish status:', 'required'=>true, 'disableLoadDefaultDecorators'=>true));
		$status->addMultiOptions(array('published'=>'Published', 'ready'=>'Ready'));
		$status->addDecorator('Label')->addDecorator('ViewHelper')->addDecorator('HtmlTag', array('tag'=>'p'));
		$this->addElement($status);
		
		$this->addDisplayGroup(
			array('datetime', 'status'),
			'publishing',
			array(
				'legend' => 'Publishing options'
			)
		);
		
		$saveButton = new Zend_Form_Element_Submit('save', array('label'=>'Save', 'class'=>'button', 'disableLoadDefaultDecorators'=>true, 'decorators'=>array('ViewHelper')));
		$publishButton = new Zend_Form_Element_Submit('publish', array('label'=>'Publish', 'class'=>'button', 'disableLoadDefaultDecorators'=>true, 'decorators'=>array('ViewHelper')));
		$this->addElements(array($saveButton, $publishButton));
		
		$this->addDisplayGroup(
			array('save', 'publish'),
			'buttons',
			array(
				'disableLoadDefaultDecorators'=>true,
				'decorators' => array(
					'FormElements',
					new Zend_Form_Decorator_HtmlTag(array('tag'=>'p', 'id'=>'buttons'))
				)
			)
		);
		
		$this->setAction($_SERVER['REQUEST_URI'])
			->setAttrib('id', 'form_' . $this->_type)
			->setAttrib('class', 'alert_form')
			->addDecorator('FormElements')
			->addDecorator('Form');
			
		//Setting instance of this form into registry, so it can be accessed from Admin_Model_WebfolioEntries model for example.
		Zend_Registry::set(self::REGISTRY_KEY, $this);
		
		parent::__construct($options);
	}
	
	/**
	 * Initializing WFEntry form plugins, by adding supplied form 
	 * elements and display groups.
	 *
	 * @return void
	 */
	protected function initWFEntryFormPlugins()
	{
		foreach (Admin_Model_WFEntryFormPlugins::get() as $WFEntryPluginData) {
			if (
				(!array_key_exists('formPick', $WFEntryPluginData) && !array_key_exists('formOmit', $WFEntryPluginData))
				|| (array_key_exists('formPick', $WFEntryPluginData) && in_array($this->_type, $WFEntryPluginData['formPick'])) || (array_key_exists('formOmit', $WFEntryPluginData) && !in_array($this->_type, $WFEntryPluginData['formOmit']))
			) {
				$this->getView()->addScriptPath(APPLICATION_PATH . '/modules/' . $WFEntryPluginData['module'] . '/views/parts'); //Adding path to module's partial scripts.
				
				foreach ($WFEntryPluginData['elements'] as $element) {
					$this->addElement($element['type'], $element['name'], $element['options']);
					$this->_pluginsElements[] = $element['name'];
				}
				
				if (array_key_exists('displayGroups', $WFEntryPluginData)) { //Has its own display group(s)?
					foreach ($WFEntryPluginData['displayGroups'] as $displayGroup) {
						$this->addDisplayGroup($displayGroup['elements'], $displayGroup['name'], $displayGroup['options']);
					}
				}
				elseif (!empty($this->_pluginsElements)) {
					if (($displayGroup = $this->getDisplayGroup($WFEntryPluginData['displayGroupName'])) != null) { //Adding element(s) into some existing display group?
						$displayGroup->addElements($this->_pluginsElements);
					}
					else {
						$this->addDisplayGroup(
							$this->_pluginsElements,
							(string)$WFEntryPluginData['displayGroupName'],
							array( 
								'legend' =>(string)$WFEntryPluginData['displayGroupLegend']
							)
						);
					}
				}
			}
		}
	}

    /**
     * Defined by Zend_Form.
     *
     * Preventing reseting of "class" attribute.
     *
     * @param  string $key
     * @param  mixed $value
     * @return Zend_Form
     */
    public function setAttrib($key, $value)
    {
        if ($key == 'class' && $this->_attribs['class']) {
            $value = $this->_attribs['class'] . ' ' . $value;
        }

        return parent::setAttrib($key, $value);
    }
	
	/**
	 * This method should be used by classes which extend this class, 
	 * in order to remove some elements which are not needed for 
	 * some webfolio entry type, and also prevents removal of 
	 * mandatory form elements and plugin elements, that were added 
	 * by some additional modules.
	 *
     * @param array Names of elements to be omitted from removal process. 
     * @return boolean
     */
    public function removeAllBut(array $omit = array())
    {
		$elements = array_merge($omit, array('title', 'text', 'save', 'id', 'titleUrlFormat'), $this->_pluginsElements);
		
		foreach ($this->_elements as $name=>$element) {
			if (!in_array($name, $elements)) {
				$this->removeElement($name);
			}
		}
    }
	
	/**
     * Defined by Zend_Form.
     * 
     * @param array $values 
     * @return Zend_Form
     */
    public function populate(array $values)
    {
        parent::populate($values);
		
		WebfolioCMS_Events_Broker::notifyEventHandlers(WebfolioCMS_Events_Broker::WF_ENTRY_FORM_POPULATE, array($this));
		
		return $this;
    }
	
	/**
	 * Defined by Zend_Form.
	 *
	 * Injecting custom pre-render logic.
	 *
	 * @param Zend_View_Interface $view 
	 * @return string
	 */
	public function render(Zend_View_Interface $view = null)
	{
		WebfolioCMS_Events_Broker::notifyEventHandlers(WebfolioCMS_Events_Broker::WF_ENTRY_FORM_PRE_RENDER, array($this));
		
		if (($hierarchyPosition = $this->getDisplayGroup('hierarchyPosition')) != null && ($id = $this->getElement('id')) != null) {
			$hierarchyPosition->getDecorator('viewScript')->setOption('id', $id->getValue());
		}
		
		if ($this->getElement('status') && $this->getElement('status')->getValue() == 'published') { 
			$this->removeElement('publish');
		}
		
		if ($this->isErrors() && strstr($this->getAttrib('class'), 'alert_form') !== false) { 
			$this->getView()->headScript(
				Zend_View_Helper_HeadScript::SCRIPT, 
				"$.protectData.protect();"
			);
		}
		
		return parent::render($view);
	}
	
	/**
	 * Defined by Zend_Form.
	 *
	 * Injecting custom pre-validation logic.
	 *
	 * @param array Submitted form data.
	 * @return bool
	 */
	public function isValid($data)
	{
		WebfolioCMS_Events_Broker::notifyEventHandlers(WebfolioCMS_Events_Broker::WF_ENTRY_FORM_PRE_VALID, array($this, $data));
		
		if (isset($data['treePosition']) && $data['treePosition'] != null) {
			$this->getElement('parentEntry')->setRequired(true);
		}
		
		return parent::isValid($data);
	}
}
?>