<?php
/**
 * Copyright (C) 2009 Nikola Posa (http://www.nikolaposa.in.rs)
 *
 * This program is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 3 of the License, or 
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
 
/**
 * Model which act as a handler for comments-related events.
 *
 * @author Nikola Posa <posa.nikola@gmail.com>
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License
 */
class Admin_Model_CommentsHookHandler
{
	const AWAITING = 'awaiting';
	const SPAM 	   = 'spam';
	
	/**
     * Webfolio settings that are used in this model. 
	 *
     * @var array
     */
	protected $settings = array();
	
	/**
     * Messages that will be outputed in comment_form AJAX handler, 
	 * in case comment's status is changed into whether 'awaiting' 
	 * or 'spam'. 
	 *
     * @var array
     */
	protected $messages = array(
        self::AWAITING => 'commentAwaitingMsg',
        self::SPAM     => 'commentSpamMsg'
    );
	
	/**
     * Initializing handler.
	 *
     * @return void
     */
    public function  __construct()
    {
		$this->settings = Model_WebfolioSettings::getSettings(
			array(
				'admin_moderate_comment', 
				'comments_notify',
				'use_akismet', 
				'akismet_api_key', 
				'site_url',
				'admin_email'
			)
		);
    }
	
	/**
     * Handles WFCommentPreSave event.
	 *
	 * @param Model_WebfolioComments Submitted comment.
     * @return void
     */
    public function WFCommentPreSave(Model_WebfolioComments $comment)
    {
		if ((bool)$this->settings['admin_moderate_comment'] == true && !WebfolioCMS_Common::isUserLoggedIn()) { //Whether comment will wait for moderation.
			$comment->status = self::AWAITING;
		}
		
		if ((bool)$this->settings['use_akismet'] == true && $this->isSpam($comment)) { //Spam checking.
			$comment->status = self::SPAM;
		}
    }
	
	/**
     * Handles WFCommentPostSave event.
	 *
	 * @param Model_WebfolioComments Comment that was just inserted in database.
     * @return void
     */
    public function WFCommentPostSave(Model_WebfolioComments $comment)
    {
		if ((bool)$this->settings['comments_notify'] == true && !WebfolioCMS_Common::isUserLoggedIn() && ($comment->status == 'approved' || $comment->status == self::AWAITING)) { //If notification needs to be sent to admin.
			$this->sendNotificationEmail($comment);
		}
		
		if (array_key_exists($comment->status, $this->messages)) {
			$output = array();
			
			$output['status'] = ($comment->status == self::AWAITING) ? 'green' : 'red';
			
			$translateAdapter = Zend_Registry::get(WebfolioCMS_Application_Resource_Translate::DEFAULT_REGISTRY_KEY);
			$output['message'] = $translateAdapter->translate($this->messages[$comment->status]);
			
			echo Zend_Json_Encoder::encode($output);  //We can do this, because we're in AJAX request.
			exit();
		}
    }
	
	/**
	 * Checking whether submitted comment is spam, using Akismet API.
	 *
	 * @return bool
	 */
	protected function isSpam(Model_WebfolioComments $comment)
	{
		$akismet = new Zend_Service_Akismet($this->settings['akismet_api_key'], $this->settings['site_url']);
		$akismetData = array(
			'user_ip'			   => $comment->authorIp, 
			'user_agent'		   => $comment->authorBrowser,
			'comment_type'		   => 'comment',
			'comment_author'	   => $comment->authorName,
			'comment_author_email' => $comment->authorEmail,
			'comment_content'	   => $comment->text
		);
		
		return $akismet->isSpam($akismetData);
	}
	
	/**
	 * Notifies admin by email, that new comment is posted on his 
	 * site.
	 *
	 * @param int|null Id of a last inserted comment.
	 * @return void
	 */
	protected function sendNotificationEmail(Model_WebfolioComments $comment)
	{
		$webfolioItemTitle = $this->getEntryTitle($comment->WFEntryId);
		
		if ($comment->status == self::AWAITING) {
			$subject = 'Please moderate comment';
			
			$urlHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('Url');
			$editCommentUrl = $this->settings['site_url'] . $urlHelper->url(array('comment'=>$comment->id), 'admin_comments_edit', true);
			
			$text = '
			<p>A new comment has been posted in "' . $webfolioItemTitle . '" and is now waiting for your approval.</p>
		
			<p>Go to your administration panel and moderate comment: <a href = "' . $editCommentUrl . '">' . $editCommentUrl . '</a></p>';
		}
		else {
			$subject = 'Comment has been posted in "' . $webfolioItemTitle . '"';
			
			$text = '
			<p>A new comment has been posted in "' . $webfolioItemTitle . '".</p>';
		}
		
		$text .= '
		<p>
			Author: ' . $comment->authorName . ' <br />
			E-mail: ' . $comment->authorEmail . ' <br />
			URL: ' . $comment->authorUrl . '<br />
			Comment: <br /> ' . $comment->text . '
		</p>
		';
		
		$webfolioMail = new WebfolioCMS_Mail('utf-8');
		@ $webfolioMail->addTo($this->settings['admin_email'])->setSubject($subject)->setBodyHtml($text)->send();
	}
	
	/**
	 * Retrieves title of some entry, based on its id.
	 *
	 * @param int Webfolio entry id.
	 * @return string|null
	 */
	protected function getEntryTitle($WFEntryId)
	{	
		$dbHandler = Zend_Registry::get('dbHandler');
		
		$sql = 'SELECT title FROM wf_entries WHERE id = ' . $dbHandler->quote($WFEntryId, Zend_DB::INT_TYPE);
		$result = $dbHandler->fetchRow($sql);
		
		if ($result) {
			return $result['title'];
		}
		else {
			return null;
		}
	}
}

