<?php
/**
 * Webfolio CMS
 *
 * Copyright (C) 2009 Nikola Posa (http://www.nikolaposa.in.rs)
 *
 * This file is part of Webfolio CMS.
 *
 * Webfolio CMS is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 3 of the License, or 
 * (at your option) any later version.
 *
 * Webfolio CMS is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with Webfolio CMS. If not, see <http://www.gnu.org/licenses/>.
 */
 
/**
 * Model for getting info about modules, in app/modules 
 * directory. This model also contains logic activating, 
 * deactivating and deleting some module.
 *
 * @author Nikola Posa <posa.nikola@gmail.com>
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License
 */
class Admin_Model_DirContent_Modules extends Admin_Model_DirContent_Abstract
{	
	const README_FILE = 'readme.txt';
	
	/**
	 * Path to modules directory.
	 *
	 * @var string
	 */
	protected $path = 'modules';
	
	/**
	 * List of modules that are active.
	 *
	 * @var array
	 */
	protected $activeModules;
	
	/**
	 * Model for installing modules.
	 *
	 * @var Admin_Model_DirContent_Modules_Install
	 */
	protected $modulesInstall;
	
	/**
	 * Constructor.
	 *
	 * @param string Path to directory.
	 * @return Admin_Model_DirContent_Abstract
	 */
	public function __construct($path = null)
	{
		$this->path = APPLICATION_PATH . DIRECTORY_SEPARATOR . $this->path;
		
		parent::__construct($path);
		
		$this->activeModules = Model_WebfolioSettings::getSettings('active_modules');
		
		$this->modulesInstall = new Admin_Model_DirContent_Modules_Install($this);
	}
	
	/**
	 * Gets info of all modules in app/modules directory (except 
	 * admin).
	 *
	 * @return array
	 */
	public function getModulesInfo()
	{
		$modulesInfo = array();
		
		foreach (new DirectoryIterator($this->path) as $content) {
			$moduleDir = $content->getFilename();
			if (!$content->isDot() && $content->isDir() && !in_array($moduleDir, array('admin', 'taxonomy'))) { //We don't want to show core modules.
				$modulesInfo[] = array_merge(
					$this->getModuleInfo($moduleDir), 
					array(
						'dirName' => $moduleDir,
						'active'  => (in_array($moduleDir, $this->activeModules)) ? 1 : 0,
                        'offerUpdate' => (($updateModel = $this->_getUpdateModel($moduleDir)) && !$updateModel->isUpdated())
					)
				);
			}
		}
		
		//Making sure that active modules come first.
		if (!empty($modulesInfo)) {
			$active = array();
			foreach ($modulesInfo as $key=>$module) {
				$active[$key] = $module['active'];
			}
			
			array_multisort($active, SORT_DESC, $modulesInfo);
		}
		
		return $modulesInfo;
	}
	
	/**
	 * Gets info of some module, from its readme.txt file.
	 *
	 * @param string Dir name of some module.
	 * @param string Name of the file which has info (readme.txt by default).
	 * @return array
	 */
	public function getModuleInfo($moduleDir)
	{
		$moduleInfo = array('name'=>'', 'uri'=>'', 'description'=>'', 'version'=>'', 'author'=>'No info.');
		
		$redmeFileName = null;
		foreach (@ scandir($this->path . DIRECTORY_SEPARATOR . $moduleDir) as $file) {
			if (preg_match('/^' . preg_quote(self::README_FILE) . '$/i', $file, $subPattern)) {
				$redmeFileName = $subPattern[0];
				break;
			}
		}
		
		if ($redmeFileName != null) {
			$readmePath = $this->path . DIRECTORY_SEPARATOR . $moduleDir . DIRECTORY_SEPARATOR . $redmeFileName;
			
			@ $readmeContent = file_get_contents($readmePath);
			
			if ($readmeContent !== false) {
				//Getting name of the module.
				if (preg_match('/Module\sname\:\s?(.+)/i', $readmeContent, $moduleName)) {
					$moduleInfo['name'] = $moduleName[1];
				}
				
				//Getting URI of the module.
				if (preg_match('/Module\sURI\:\s?((https?:\/\/)?(www\.)?[a-z0-9\.\-]+\.[a-z]{2,6}(\/?[a-z0-9_\-\.%\?&=]*)*)/i', $readmeContent, $moduleUri)) {
					$uri = $moduleUri[1];
					
					if (strpos($uri, '://') === false) {
						$uri = 'http://' . $uri;
					}
					
					$moduleInfo['uri'] = $uri;
				}
				
				//Getting module's description.
				if (preg_match('/Description\:\s?(.+)/i', $readmeContent, $description)) {
					$moduleInfo['description'] = $this->prepareModuleDescription($description[1], 70);
				}
				
				//Getting module's version.
				if (preg_match('/Version\:\s?(.+)/i', $readmeContent, $version)) {
					$moduleInfo['version'] = $version[1];
				}
				
				//Getting module's author(s) info.
				$authorsName = array();
				if (preg_match('/Author\:\s?(.+)/i', $readmeContent, $author)) {
					$authors = $author[1];
					
					$authorsName = explode(',', $authors);
				}
				
				$authorsUri = array();
				if (preg_match('/Author\sURI\:\s?(.+)/i', $readmeContent, $authorUri)) {
					$authorsUriList = $authorUri[1];
					
					$authorsUri = explode(',', $authorsUriList);
					
					foreach ($authorsUri as $key=>&$uri) {
						$uri = trim($uri);
						
						if (!preg_match('/(https?:\/\/)?(www\.)?[a-z0-9\.\-]+\.[a-z]{2,6}(\/?[a-z0-9_\-\.%\?&=]*)*/i', $uri)) {
							unset($authorsUri[$key]);
						}
						elseif (strpos($uri, '://') === false) {
							$uri = 'http://' . $uri;
						}
					}
				}
				
				if (!empty($authorsName)) {
					$authors = array();
					foreach ($authorsName as $key=>$name) {
						$name = trim($name);
						
						if (isset($authorsUri[$key])) {
							$authors[] = '<a href = "' . $authorsUri[$key] . '">' . $name . '</a>';
						}
						else {
							$authors[] = $name;
						}
					}
					
					$moduleInfo['author'] = implode(', ', $authors);
				}
			}
		}
		
		return $moduleInfo;
	}
	
	/**
	 * Limites text description of some module in case that it has 
	 * more that $limit characters in one line, breaking text into 
	 * lines.
	 *
	 * @param string Description.
	 * @param int On how many characters should we break into new line (<br /> tag).
	 * @return string
	 */
	protected function prepareModuleDescription($description, $limit)
	{
		if (strlen($description) > $limit) {
			$lastSpace = strpos($description, ' ', $limit);
			if ($lastSpace !== false) {
				$firstPart = substr($description, 0, $lastSpace);
				$secondPart = substr($description, $lastSpace);
				
				$description = $firstPart . '<br />' . $this->prepareModuleDescription($secondPart, $limit);
			}
		}
		
		return $description;
	}
	
	/**
	 * This function does the activation of some module(s). Beside 
	 * activation, module will also be installed if this is the 
	 * first time user is activating it. Not only that module will 
	 * be added into active_modules list in database, but it will 
	 * also move all front-end resources (ajax_handlers, css, img, 
	 * js) of some module (if any exist) into front-end directory.
	 *
	 * @param mixed Name of the module.
	 * @return void
	 */
	public function activateModule($module)
	{
		$moduleDirs = $this->prepareModules($module);
		
		foreach ($moduleDirs as $moduleDir) {
			if (is_dir($this->path . DIRECTORY_SEPARATOR . $moduleDir) && !in_array($moduleDir, $this->activeModules)) {
				$this->modulesInstall->installModule($moduleDir);
				
				//Activating module.
				$this->activeModules[] = $moduleDir;
				Model_WebfolioSettings::updateSetting('active_modules', $this->activeModules);
			}
		}
	}
	
	/**
	 * Deactivates some module(s).
	 *
	 * @param mixed Name of the module.
	 * @return void
	 */
	public function deactivateModule($module)
	{
		$moduleDirs = $this->prepareModules($module);
		
		foreach ($moduleDirs as $moduleDir) {
			if (in_array($moduleDir, $this->activeModules)) { //Only if module is activated.
				foreach ($this->activeModules as $key=>$module) {
					if ($module == $moduleDir) {
						unset($this->activeModules[$key]);
						sort($this->activeModules);
						
						break;
					}
				}
				
				Model_WebfolioSettings::updateSetting('active_modules', $this->activeModules);
			}
		}
	}
	
	/**
	 * Deletes some module(s), including both module's files and db 
	 * data.
	 *
	 * @param mixed Name of the module.
	 * @return void
	 */
	public function deleteModule($module)
	{
		$moduleDirs = $this->prepareModules($module);
		
		foreach ($moduleDirs as $moduleDir) {
			if (!in_array($moduleDir, $this->activeModules)) { //Only if module is NOT activated.
				//Deleting module's folder.
				$path = $this->path . DIRECTORY_SEPARATOR . $moduleDir;
				
				@ chmod($path, 0777);
				
				foreach (new RecursiveIteratorIterator(new RecursiveDirectoryIterator($path), RecursiveIteratorIterator::CHILD_FIRST) as $file) {
					if ($file->isDir()) {  
						@ rmdir($file->getPathname());  
					} 
					else {  
						@ unlink($file->getPathname());  
					} 
				}
				
				@ rmdir($path);
				
				$this->modulesInstall->uninstallModule($moduleDir);
			}
		}
	}

    /**
	 * Updates db-related logic of some module.
	 *
	 * @param string Name of the module.
	 * @return void
	 */
    public function updateModule($module)
    {
        $moduleDirs = $this->prepareModules($module);

        foreach ($moduleDirs as $module) {
            $updateModel = $this->_getUpdateModel((string)$module);

            if (!$updateModel->isUpdated()) {
                $updateModel->update();
            }
            else {
                throw new Exception($module . ' module is already up to date.');
            }
        }
    }

    /**
	 * This function is used by activateModule(),
	 * deactivateModule() and deleteModule() functions, in order
	 * to be sure that array of module names is passed to foreach
	 * loop.
	 *
	 * @param mixed Name of the module.
	 * @return array
	 */
	protected function prepareModules($module)
	{
		$moduleDirs = $module;

		if (!is_array($moduleDirs)) {
			$moduleDirs = array($moduleDirs);
		}

		return $moduleDirs;
	}

    /**
	 * Retrieves update model instance of some module (if such
     * exists).
	 *
	 * @param string Name of the module.
	 * @return Admin_Model_Update_Abstract|null
	 */
    protected function _getUpdateModel($module)
    {
        $classPrefix = str_replace(array('-', '.'), ' ', strtolower($module));
        $classPrefix = ucwords($classPrefix);
        $classPrefix = str_replace(' ', '', $classPrefix);

        $className = $classPrefix . '_Model_Update'; //Class name must have this scheme.
        
        if (class_exists($className)) {
            $updateModel = new $className();
            if ($updateModel instanceof Admin_Model_Update_Abstract) {
                return $updateModel;
            }
        }

        return null;
    }
}
?>