<?php
/**
 * Webfolio CMS
 *
 * Copyright (C) 2009 Nikola Posa (http://www.nikolaposa.in.rs)
 *
 * This file is part of Webfolio CMS.
 *
 * Webfolio CMS is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 3 of the License, or 
 * (at your option) any later version.
 *
 * Webfolio CMS is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with Webfolio CMS. If not, see <http://www.gnu.org/licenses/>.
 */
 
/**
 * Model that has logic for installing/uninstalling some module.
 *
 * @author Nikola Posa <posa.nikola@gmail.com>
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License
 */
class Admin_Model_DirContent_Modules_Install extends Model_WebfolioBase
{
	const DB_TABLES = 'dbTables';
	const WF_SETTINGS = 'WFSettings';
	
	/**
	 * Modules directory content.
	 *
	 * @var WebfolioCMS_DirContent
	 */
	protected $modulesDir;
	
	/**
	 * List of modules that are already installed.
	 *
	 * @var array
	 */
	protected $installedModules;
	
	/**
	 * Constructor.
	 *
	 * @return Admin_Model_DirContent_Modules_Install
	 */
	public function __construct(Admin_Model_DirContent_Modules $modulesDir)
	{
		$this->modulesDir = $modulesDir;
	
		$this->installedModules = Model_WebfolioSettings::getSettings('installed_modules');
		
		parent::__construct();
	}
	
	/**
	 * Installs some module, moving its front-end resources into 
	 * front-end dir, and installing its db tables and settings if 
	 * it has any of those.
	 *
	 * @param string Name of the module.
	 * @return void
	 */
	public function installModule($moduleName)
	{
		if (!in_array($moduleName, array_keys($this->installedModules))) { //Module's not already installed?
			$this->moveFrontEndResources($moduleName);

            $modulesDbData = $this->dbInstall($moduleName);

			//Updating installed modules info.
			$this->installedModules[$moduleName] = array(self::DB_TABLES=>$modulesDbData[self::DB_TABLES], self::WF_SETTINGS=>$modulesDbData[self::WF_SETTINGS]);
			Model_WebfolioSettings::updateSetting('installed_modules', $this->installedModules);
		}
	}
	
	/**
	 * Moving all front-end resources (ajax_handlers, css, img, js) 
	 * of some module (if has any) into front-end directory. In 
	 * every of those resource folders, separate dir is created for 
	 * some module.
	 *
	 * @param string Name of the module.
	 * @return void
	 */
	protected function moveFrontEndResources($moduleDirName)
	{
		$path = $this->modulesDir->getPath() . DIRECTORY_SEPARATOR . $moduleDirName . DIRECTORY_SEPARATOR . 'front-end';
		
		if (is_dir($path)) { //Has some front-end resources?
			foreach (new RecursiveIteratorIterator(new RecursiveDirectoryIterator($path), RecursiveIteratorIterator::CHILD_FIRST) as $file) {	
				if ($file->isDir()) { //If we reached to some dir, we should remove it.
					@ rmdir($file->getPathname());  
				} 
				else {  //File...
					$dir = substr($file->getPath(), strrpos($file->getPath(), DIRECTORY_SEPARATOR)+1); //js, img, css...
					
					$pathToResources = BASE_PATH . DIRECTORY_SEPARATOR . 'front-end' . DIRECTORY_SEPARATOR . $dir . DIRECTORY_SEPARATOR . $moduleDirName; //Path to some front-end resources.
					if (!file_exists($pathToResources)) {
						mkdir($pathToResources, 0777);
					}
				
					if (!file_exists($pathToResources . DIRECTORY_SEPARATOR . $file->getFilename())) {
						if (rename($file->getPathname(), $pathToResources . DIRECTORY_SEPARATOR . $file->getFilename()) == false) {
							throw new Exception('An error occurred in module activation process. You should manually move all front-end resources of module you\'re trying to activate into appropriate dir. Read the instructions for more info.');
						}
					}  
				} 
			}
			
			@ rmdir($path);
		}
	}
	
	/**
	 * Installing db tables and settings of some module, if 
	 * it has any of those.
	 *
	 * @param string Name of the module.
	 * @return array
	 */
	protected function dbInstall($moduleName)
	{
		$moduleDbTables = array();
		$moduleWFSettings = array();
			
		$path = $this->modulesDir->getPath() . DIRECTORY_SEPARATOR . $moduleName . DIRECTORY_SEPARATOR . 'db_install.php';
		
		if (file_exists($path)) { //Has some db install script?
			$dbInstallData = include $path;
			
			if (isset($dbInstallData[self::DB_TABLES])) {
				$moduleDbTables = array_keys($dbInstallData[self::DB_TABLES]); //Getting names of module's db tables, so we can delete them in deleting process.
				
				foreach ($dbInstallData[self::DB_TABLES] as $sql) {
					$this->dbHandler->query($sql);
				}
			}
			
			if (isset($dbInstallData[self::WF_SETTINGS])) {
				$moduleWFSettings = array(); 
				
				foreach ($dbInstallData[self::WF_SETTINGS] as $settingName=>$data) {
					$settingName = $moduleName . '_' . $settingName; //Name of the module's settings should be prefixed with its name.
					$autoload = ($data['autoload']) ? $data['autoload'] : 0;
					
					Model_WebfolioSettings::addSetting($settingName, $data['value'], $autoload);
					
					$moduleWFSettings[] = $settingName; //Getting names of module's settings, so we can delete them in deleting process.
				}
			}
		}
		
		return array(self::DB_TABLES=>$moduleDbTables, self::WF_SETTINGS=>$moduleWFSettings);
	}
	
	/**
	 * Uninstalls some module, removing its front-end resources and 
	 * deleting its db tables and settings if it has any of those.
	 *
	 * @param string Name of the module.
	 * @return void
	 */
	public function uninstallModule($moduleName)
	{
		$this->removeFrontEndResources($moduleName);
		
		$this->dbUninstall($moduleName);
	}
	
	/**
	 * Deleting all front-end resources (ajax_handlers, css, img, 
	 * js) of some module (if any exists).
	 *
	 * @param string Name of the module.
	 * @return void
	 */
	protected function removeFrontEndResources($moduleDirName)
	{
		$path = BASE_PATH . DIRECTORY_SEPARATOR . 'front-end';
		
		@ chmod($path, 0777);
		
		foreach (new RecursiveIteratorIterator(new RecursiveDirectoryIterator($path), RecursiveIteratorIterator::CHILD_FIRST) as $file) {
			$path = $file->getPathname();
			
			if ($file->isDir() && substr($path, strrpos($path, DIRECTORY_SEPARATOR)+1) == $moduleDirName) {  
				@ chmod($path, 0777);
				@ rmdir($path);  
			} 
			elseif ($file->isFile() && substr($file->getPath(), strrpos($file->getPath(), DIRECTORY_SEPARATOR)+1) == $moduleDirName) {  
				@ unlink($file->getPathname());  
			} 
		}
	}
	
	/**
	 * Deleting db tables and settings of some module, if it has 
	 * any of those.
	 *
	 * @param string Name of the module.
	 * @return void
	 */
	protected function dbUninstall($moduleName)
	{
		if (array_key_exists($moduleName, $this->installedModules)) { //Module is installed?
			if (!empty($this->installedModules[$moduleName][self::DB_TABLES])) { //Has some db tables?
				foreach ($this->installedModules[$moduleName][self::DB_TABLES] as $dbTableName) {
					$this->dbHandler->query('DROP TABLE IF EXISTS ' . $this->dbHandler->quoteIdentifier($dbTableName));
				}
			}
			
			if (!empty($this->installedModules[$moduleName][self::WF_SETTINGS])) { //Has some webfolio settings?
				Model_WebfolioSettings::deleteSettings($this->installedModules[$moduleName][self::WF_SETTINGS]);
			}
			
			//Updating installed modules info.
			unset($this->installedModules[$moduleName]);
			Model_WebfolioSettings::updateSetting('installed_modules', $this->installedModules);
		}
	}
	
}
?>