<?php
/**
 * Webfolio CMS
 *
 * Copyright (C) 2009 Nikola Posa (http://www.nikolaposa.in.rs)
 *
 * This file is part of Webfolio CMS.
 *
 * Webfolio CMS is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 3 of the License, or 
 * (at your option) any later version.
 *
 * Webfolio CMS is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with Webfolio CMS. If not, see <http://www.gnu.org/licenses/>.
 */
 
/**
 * Model for getting info about themes in front-end/themes 
 * directory.
 *
 * @author Nikola Posa <posa.nikola@gmail.com>
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License
 */
class Admin_Model_DirContent_Themes extends Admin_Model_DirContent_Abstract
{
	/**
	 * Constructor.
	 *
	 * @param string Path to directory.
	 * @return Admin_Model_DirContent_Abstract
	 */
	public function __construct($path = null)
	{
		$this->path = BASE_PATH . DIRECTORY_SEPARATOR . 'front-end' . DIRECTORY_SEPARATOR . 'themes';
		
		parent::__construct($path);
	}
	
	/**
	 * Gets info of themes.
	 *
	 * @return array
	 */
	public function getThemesInfo()
	{
		$themesInfo = array();
		
		$i = 0;
		foreach (new DirectoryIterator($this->path) as $content) {
			if (!$content->isDot() && $content->isDir()) {
				$themeDir = $content->getFilename();
				@ $themeFiles = scandir($this->path . DIRECTORY_SEPARATOR . $themeDir);
				
				//We need to make sure that each theme has these files: style.css, layout.phtml, header.phtml, footer.phtml (at least).
				if (in_array('style.css', $themeFiles) && in_array('layout.phtml', $themeFiles) && in_array('header.phtml', $themeFiles) && in_array('footer.phtml', $themeFiles)) {
					//Getting theme info.
					$themeInfo = $this->getThemeInfo($themeDir);
					$themesInfo[$i] = $themeInfo;
				
					//Adding name of theme's directory into theme info.
					$themesInfo[$i]['dirName'] = $themeDir;
				
					//Adding path to screenshot image (if exists) of some theme into theme info.
					$baseUrl = WebfolioCMS_Common::baseUrl();
					$themesInfo[$i]['screenshot'] = '<img src = "' . $baseUrl . '/front-end/img/admin/no_screenshot.gif" alt = "No screenshot" />';
				
					foreach ($themeFiles as $file) {
						if (is_string($file) && substr($file, 0, 10) == 'screenshot') { //Has screenshot image?
							$themesInfo[$i]['screenshot'] = '<img src = "' . $baseUrl . '/front-end/themes/' . $themeDir . '/' . $file . '" alt = "' . $themesInfo[$i]['name'] . '" style = "width: 300px; height: 225px;" />';
							break;
						}
					}
				
					//Adding "active" flag into theme info.
					$themesInfo[$i]['active'] = 0;
					$currentTheme = Model_WebfolioSettings::getSettings('theme_dir');
					if ($currentTheme == $themeDir) {
						$themesInfo[$i]['active'] = 1;
					}
					
					$i++;
				}
			}
		}
		
		/*//Making sure that active theme comes first.
		if (!empty($themesInfo)) {
			foreach ($themesInfo as $key=>$theme) {
				$active[$key] = $theme['active'];
			}
			array_multisort($active, SORT_DESC, $themesInfo);
		}*/
		
		return $themesInfo;
	}
	
	/**
	 * Extracts theme info (name, url, description, version, author 
	 * info) from CSS file (style.css).
	 *
	 * @param string Dir name of some theme.
	 * @return array
	 */
	protected function getThemeInfo($themeDirName)
	{
		$cssFilePath = $this->path . DIRECTORY_SEPARATOR . $themeDirName . '/style.css';
		
		$themeInfo = array('name'=>'', 'uri'=>'', 'description'=>'', 'version'=>'', 'author'=>'No info.');
		
		@ $styleContent = file_get_contents($cssFilePath);
		
		if ($styleContent !== false) {
			$info = array();
			if (preg_match('/\/\*\*(.+)\*\//Us', $styleContent, $infoArray)) {
				$info = trim($infoArray[1]);
				
				//Matching theme name and URI.		
				if (preg_match('/\*\s?([^@]+)\s?<((https?:\/\/)?(www\.)?[a-z0-9\.\-]+\.[a-z]{2,6}(\/?[a-z0-9_\-\.%\?&=]*)*)>/i', $info, $themeNameURI)) {
					$themeInfo['name'] = trim($themeNameURI[1]);
					$themeInfo['uri']  = trim($themeNameURI[2]);
				}
				elseif (preg_match('/\*\s?(.+)/', $info, $themeName)) { //Matching only name of theme.
					$themeInfo['name'] = trim($themeName[1]);
				}
				
				//Matching theme description.
				if (preg_match('/\n\s\*\s?([^@]+)/s', $info, $description)) {
					$desc = str_replace('*', '', $description[1]);
					$desc = trim($desc);
					$desc = nl2br($desc);
					
					$themeInfo['description'] = $desc;
				}
				
				//Matching theme version.
				if (preg_match('/@version\s?([a-z0-9\.]+)/i', $info, $version)) {
					$themeInfo['version'] = trim($version[1]);
				}
				
				//Matching name and URI of theme's author.
				if (preg_match_all('/@author\s?(.+)/', $info, $authorNameURI)) { 
					$authors = array();
					foreach ($authorNameURI[1] as $key=>$anu) {
						if (preg_match('/<((https?:\/\/)?(www\.)?[a-z0-9\.\-]+\.[a-z]{2,6}(\/?[a-z0-9_\-\.%\?&=]*)*)>/i', $anu, $uriArray) || preg_match('/<(([a-z0-9\+_\-]+)(\.[a-z0-9\+_\-]+)*@([a-z0-9\-]+\.)+[a-z]{2,6})>/ix', $anu, $uriArray)) {
							$name = substr($anu, 0, strpos($anu, '<')-1);
							$uri = $uriArray[1];
							if (strpos($uri, '@') !== false) { //Email?
								$uri = 'mailto:' . $uri;
							}
							elseif (strpos($uri, '://') === false) { //Url without protocol?
								$uri = 'http://' . $uri;
							}
							
							$author = '<a href = "' . $uri . '">' . trim($name) . '</a>';
						}
						else {
							$author = trim($anu);
						}
						
						$authors[] = $author;
					}
					
					$themeInfo['author'] = implode(', ', $authors);
				}
			}
		}
		
		if (strlen($themeInfo['name']) == 0) { //No theme name found?
			$themeInfo['name'] = ucfirst($themeDirName);
		}
		
		return $themeInfo;
	}
	
	/**
	 * Gets theme content (names of templates and stylesheets files).
	 *
	 * @param string Theme's folder name.
	 * @return array
	 */
	public function getThemeContent($themeName)
	{
		if (!is_dir($this->path . DIRECTORY_SEPARATOR . $themeName)) {
			throw new Exception('That theme does not exist in front-end/themes folder.'); 
		}
		
		$themeContent = array();
		
		$marker = 0;
		foreach(new DirectoryIterator($this->path . DIRECTORY_SEPARATOR . $themeName) as $content) {
			if ($content->isFile()) { //We need files, not folders.
				$file = $content->getFilename();
				
				$name = substr($file, 0, strrpos($file, '.'));
				$ext = substr($file, strrpos($file, '.')+1);
				
				if ($ext == 'css') {
					$title = ($name != 'style') ? ucfirst($name) . ' stylesheet' : 'Main stylesheet';
					
					$style = array('title'=>$title, 'fileName'=>$file);
					if ($marker == 0) {
						$style['content'] = $this->loadFile($themeName, $file);
						$marker = 1;
					}
					
					$themeContent['styles'][] = $style;
				}
				elseif ($ext == 'phtml' && $name != 'layout') {
					$themeContent['templates'][] = array('title'=>ucfirst($name), 'fileName'=>$file);
				}
			}
		}
		
		return $themeContent;
	}
	
	/**
	 * Reads content of some theme's file.
	 *
	 * @param string Theme's folder name.
	 * @param string File name.
	 * @return string
	 */
	public function loadFile($themeDirName, $fileName)
	{
		$path = $this->path . DIRECTORY_SEPARATOR . $themeDirName . DIRECTORY_SEPARATOR . $fileName;
		
		if (!file_exists($path)) {
			throw new Exception('File with name ' . $fileName . ' does not exist in ' . $themeDirName . ' folder.');
		}
		
		@ $fileContent = file_get_contents($path);
		
		if (false === $fileContent) {
			throw new Exception('Content of file ' . $themeDirName . DIRECTORY_SEPARATOR . $fileName . ' could not be retrieved.'); 
		}
		
		return $fileContent;
	}
	
	/**
	 * Updates content of some theme's file.
	 *
	 * @param string Theme's folder name.
	 * @param string File name.
	 * @param string New content.
	 * @return void
	 */
	public function updateFile($themeDirName, $fileName, $fileContent)
	{
		$path = $this->path . DIRECTORY_SEPARATOR . $themeDirName . DIRECTORY_SEPARATOR . $fileName;
		
		if (!file_exists($path)) {
			throw new Exception('File with name ' . $fileName . ' does not exist in ' . $themeDirName . ' folder.');
		}
		
		if (false === file_put_contents($path, $fileContent)) {
			throw new Exception('Content of file ' . $themeDirName . DIRECTORY_SEPARATOR . $fileName . ' could not be updated.'); 
		}
	}
}
?>