<?php
/**
 * Webfolio CMS
 *
 * Copyright (C) 2009 Nikola Posa (http://www.nikolaposa.in.rs)
 *
 * This file is part of Webfolio CMS.
 *
 * Webfolio CMS is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 3 of the License, or 
 * (at your option) any later version.
 *
 * Webfolio CMS is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with Webfolio CMS. If not, see <http://www.gnu.org/licenses/>.
 */
 
/**
 * Abstract class that provides basic API for updating 
 * database revision of some part of Webfolio CMS (core, 
 * modules).
 *
 * @author Nikola Posa <posa.nikola@gmail.com>
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License
 */
abstract class Admin_Model_Update_Abstract extends Model_WebfolioBase
{
	const UPDATE_METHOD_PREFIX = 'update';
	
	/**
	 * Database revision value.
	 *
	 * @var int
	 */
	protected $dbRevision = 0;
	
	/**
	 * Name of setting entry which stores info about current 
	 * database revision.
	 *
	 * @var string
	 */
	protected $dbRevisionSettingName;
	
	/**
	 * Database revision value in user's database.
	 *
	 * @var string
	 */
	private $dbRevisionSetting;
	
	/**
	 * Constructor.
	 *
	 * @return Admin_Model_Update_Abstract
	 */
	public function __construct()
	{
		parent::__construct();
		
		if (!$this->dbRevisionSettingName) {
			throw new Exception('Setting name of database revison must be provided in order to use Update model API.');
		}
	}
	
	/**
	 * Retrieves database revision value.
	 *
	 * @return int
	 */
	public function getDbRevision()
	{
		return $this->dbRevision;
	}
	
	/**
	 * Retrieves database revision value in user's database.
	 *
	 * @return int
	 */
	public function getDbRevisionSetting()
	{
		if (!$this->dbRevisionSetting) {
			if (Model_WebfolioSettings::getSettings($this->dbRevisionSettingName) === null) {
				Model_WebfolioSettings::addSetting($this->dbRevisionSettingName, 0);
			}
			
			$this->dbRevisionSetting = (int)Model_WebfolioSettings::getSettings($this->dbRevisionSettingName);
		}
		
		return $this->dbRevisionSetting;
	}
	
	/**
	 * Checks whether some Webfolio CMS component is already up to 
	 * date.
	 *
	 * @return bool
	 */
	public function isUpdated()
	{
		return !($this->getDbRevisionSetting() < $this->dbRevision);
	}
	
	/**
	 * Does the updating process, by calling appropriate update 
	 * methods defined by child class, that has db revision prefix 
	 * larger than current db revision in user's database.
	 *
	 * @return void
	 */
	public function update()
	{
		if (!$this->isUpdated()) {
			foreach (get_class_methods($this) as $methodName) {
				if (substr($methodName, 0, 6) == self::UPDATE_METHOD_PREFIX && (int)substr($methodName, 6) > $this->getDbRevisionSetting()) {
					$this->$methodName();
				}
			}
			
			Model_WebfolioSettings::updateSetting($this->dbRevisionSettingName, $this->dbRevision);
		}
	}
}
?>