<?php
/**
 * Webfolio CMS
 *
 * Copyright (C) 2009 Nikola Posa (http://www.nikolaposa.in.rs)
 *
 * This file is part of Webfolio CMS.
 *
 * Webfolio CMS is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 3 of the License, or 
 * (at your option) any later version.
 *
 * Webfolio CMS is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with Webfolio CMS. If not, see <http://www.gnu.org/licenses/>.
 */
 
/**
 * Model for updating Webfolio CMS database schema and 
 * settings.
 *
 * @author Nikola Posa <posa.nikola@gmail.com>
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License
 */
class Admin_Model_Update_Core extends Admin_Model_Update_Abstract
{
	/**
	 * Database revision value.
	 *
	 * @var int
	 */
	public $dbRevision = 8;
	
	/**
	 * Name of setting entry which stores info about current 
	 * database revision.
	 *
	 * @var string
	 */
	protected $dbRevisionSettingName = 'db_revision';
	
	/**
	 * Updating database to 1. revision.
	 *
	 * @return void
	 */
	protected function update1()
	{
		Model_WebfolioSettings::addSetting('update_core_info', array('last_checked'=>0, 'version'=>WebfolioCMS_Version::VERSION));
	}
	
	/**
	 * Updating database to 2. revision.
	 *
	 * When support for RSS feeding was added in Webfolio CMS v1.0.4, 
	 * a new option, for choosing whether or not RSS feed of webfolio 
	 * items should be enabled, is available for user.  
	 *
	 * @return void
	 */
	protected function update2()
	{
		$this->update1(); //Because of bug in previous version.
		
		Model_WebfolioSettings::addSetting('enable_rss_wf_items', '1');
	}
	
	/**
	 * Updating database to 3. revision.
	 *
	 * Setting which holds content plugins scheme now has new key 
	 * ("settings"), which holds both selections and topSeparator 
	 * settings of some plugin, so we need to change structure of 
	 * that array.
	 *
	 * @return void
	 */
	protected function update3()
	{
		$contentPlugins = Model_WebfolioSettings::getSettings('content_plugins');
		
		$shouldUpdate = false;
		
		//Updating new content plugins scheme. "Settings" key was added in version 1.0.5, and it merged "selections" and "topSeparator" options.
		foreach ($contentPlugins as $type=>$plugins) {
			foreach ($plugins as $i=>$plugin) {
				if (!isset($plugin['settings'])) {
					$contentPlugins[$type][$i]['settings'] = array(
						'selections'   => $plugin['selections'],
						'topSeparator' => $plugin['topSeparator']
					);
					
					unset($contentPlugins[$type][$i]['selections']);
					unset($contentPlugins[$type][$i]['topSeparator']);
					
					$shouldUpdate = true;
				}
			}
		}
		
		if ($shouldUpdate == true) {
			Model_WebfolioSettings::updateSetting('content_plugins', $contentPlugins);
		}
	}
	
	/**
	 * Updating database to 4. revision.
	 *
	 * short_desc column in wf_entries table is not needed any more.
	 *
	 * @return void
	 */
	protected function update4()
	{
		$wfEntriesDbTable = new Model_DbTable_WebfolioEntries();
		
		//We don't need short_desc column any more.
		$sql = "ALTER TABLE " . $wfEntriesDbTable->info(Model_DbTable_WebfolioEntries::NAME) . " DROP COLUMN short_desc"; 
		
		$wfEntriesDbTable->getAdapter()->query($sql);
	}
	
	/**
	 * Updating database to 5. revision.
	 *
	 * Column which holds IP address of person who submitted comment, 
	 * in wf_comments table, is now stored as unsigned integer type. 
	 *
	 * @return void
	 */
	protected function update5()
	{
		$wfCommentsDbTable = new Model_DbTable_WebfolioComments();
		
		//Converting all IP addresses to integer values using INET_ATON() function.
		$wfCommentsDbTable->update(array('author_ip'=>new Zend_Db_Expr('INET_ATON(author_ip)')), '');
		
		//Modifying data type of "author_ip" column.
		$sql = "ALTER TABLE " . $wfCommentsDbTable->info(Model_DbTable_WebfolioEntries::NAME) . " MODIFY author_ip INT unsigned NOT NULL";
		$wfCommentsDbTable->getAdapter()->query($sql);
	}

    /**
	 * Updating database to 6. revision.
	 *
	 * Due to implementation of "Remember me" functionality, an marker
     * db setting is needed, which is used in AdminAuthChecker plugin
     * and LoginController, to be as a signal for AdminAuthChecker,
     * which calls Zend_Session::rememberMe() on every request, if user
     * checked "Stay signed in" durgin login process.
	 *
	 * @return void
	 */
	protected function update6()
	{
		Model_WebfolioSettings::addSetting('remember_me_marker', '0', 1);
	}

    /**
	 * Updating database to 7. revision.
	 *
	 * Changes in database scheme due to implementation of multi-user
     * functionality.
	 *
	 * @return void
	 */
	protected function update7()
	{
        //Fetching id of a current Webfolio CMS user, to be DEFAULT value
        //for user_id column, which will be added to wf_entries db table.
        $wfUsers = new Model_WebfolioUsers();
        $user = $wfUsers->fetchRow(null, 'id ASC');

        //Adding user_id column in wf_entries db table.
        $wfEntriesTable = new Model_DbTable_WebfolioEntries();
		$sql = "ALTER TABLE " . $wfEntriesTable->info(Model_DbTable_WebfolioEntries::NAME) . " ADD user_id int(10) unsigned NOT NULL DEFAULT " . $user['id'] . " AFTER id";
        $wfEntriesTable->getAdapter()->query($sql);

        //Adding group_id and public_name columns in wf_users db table.
		$sql = "ALTER TABLE " . $wfUsers->getMapper()->getDbTable()->info(Model_DbTable_WebfolioEntries::NAME) . " ADD group_id int(10) unsigned NOT NULL DEFAULT 0 AFTER id, ADD public_name varchar(250) NOT NULL";
        $this->dbHandler->query($sql);
        
        //Creating wf_user_meta db table.
        $this->dbHandler->query("
        CREATE TABLE IF NOT EXISTS `wf_user_meta` (
            `id` int(10) unsigned NOT NULL AUTO_INCREMENT,
            `user_id` int(10) unsigned NOT NULL,
            `key` varchar(100) NOT NULL,
            `value` text NOT NULL,
             PRIMARY KEY (`id`)
        )");

        //Creating wf_groups db table.
        $this->dbHandler->query(
        "CREATE TABLE IF NOT EXISTS `wf_groups` (
            `id` int(10) unsigned NOT NULL AUTO_INCREMENT,
            `name` varchar(100) NOT NULL,
            PRIMARY KEY (`id`)
        )");

        //Creating wf_group_acl db table.
        $this->dbHandler->query(
        "CREATE TABLE IF NOT EXISTS `wf_group_acl` (
            `id` int(10) unsigned NOT NULL AUTO_INCREMENT,
            `group_id` int(10) unsigned,
            `resource` varchar(200) NOT NULL,
            `privilege` varchar(200) DEFAULT NULL,
            `assert` text DEFAULT NULL,
            PRIMARY KEY (`id`)
        )");

		$groupsDbTable = new Model_DbTable_WebfolioGroups();
        //Populating default user groups.
        foreach (Admin_Model_Install::getInitialUsersGroups() as $groupName) {
            $groupsDbTable->insert(array('name'=>$groupName));
        }

        $groupACLDbTable = new Model_DbTable_WebfolioGroupAcl();
        //Populating default ACLs.
        foreach (Admin_Model_Install::getInitialGroupACLs() as $acl) {
            $groupACLDbTable->insert($acl);
        }

        //Fetching id of a Administrator group.
        $adminGroup = $groupsDbTable->fetchRow('name = \'Administrator\'');

        $user->publicName = $user->username; //We should set public name to be same as username.
        $user->setGroupId((int)$adminGroup['id']);
        $user->save();
	}

    /**
	 * Updating database to 8. revision.
	 *
	 * After implementing collapsible functionality for sidebar menu widget,
	 * which has its own settings, those settings should be added in
	 * "sidebar_items" webfolio settings entry.
	 *
	 * @return void
	 */
	protected function update8()
	{
        $shouldUpdate = false;

        $sidebarItems = Model_WebfolioSettings::getSettings('sidebar_items');
        foreach ($sidebarItems as $key=>$item) {
            if ($item['helperName'] == 'WFSidebarMenu' && $item['module'] == 'default' && !isset($item['settings'])) { //Sidebar menu widget?
                $sidebarItems[$key]['settings'] = array(
                    'make_collapsible' => 1,
                    'collapse_from_level'=>1,
                    'event_type'=>'click'
                );

                $shouldUpdate = true;
            }
        }

        if ($shouldUpdate == true) {
			Model_WebfolioSettings::updateSetting('sidebar_items', $sidebarItems);
		}
	}
}
?>