<?php
/**
 * Webfolio CMS
 *
 * Copyright (C) 2009 Nikola Posa (http://www.nikolaposa.in.rs)
 *
 * This file is part of Webfolio CMS.
 *
 * Webfolio CMS is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 3 of the License, or 
 * (at your option) any later version.
 *
 * Webfolio CMS is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with Webfolio CMS. If not, see <http://www.gnu.org/licenses/>.
 */
 
/**
 * Checks whether new version of Webfolio CMS is available, 
 * and generates notification message, based on some supplied 
 * string.
 *
 * @author Nikola Posa, www.nikolaposa.in.rs
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License
 */
class Zend_View_Helper_UpdateNotify extends Zend_View_Helper_Abstract
{
    /**
     * Update core info settings entry in database.
     *
     * @var string
     */
    protected $updateCoreInfo = null;
	
	/**
     * Update checking period (number of seconds).
     *
     * @var int
     */
	protected $updateCheckPeriod = 604800; //7 days.
	
	/**
     * URI on which HTTP request for fetching current project 
	 * version will be sent if necessary.
     *
     * @var string
     */
	protected $projectFilesUri = 'https://sourceforge.net/projects/webfolio-cms/files/';

    /**
     * Returns update notification message, if newer version of 
	 * Webfolio CMS is available.
     *
     * @param string Update notify message with optional placeholder for version number.
     * @return string
     */
    public function updateNotify($message)
    {
		$currentVersion = $this->getProjectVersion();
		
        if (WebfolioCMS_Version::compare($currentVersion) == 1) { //Newer version is available?
			return '<a href = "' . $this->projectFilesUri . '" rel = "nofollow">' . sprintf($message, $currentVersion) . '</a>';
		}
		else {
			return '';
		}
    }

    /**
     * Gets Webfolio CMS project current version from the 
	 * "update_core_info" settings entry. If it time to update, this 
	 * function will fetch current version from the server on which 
	 * this project is hosted.
     *
     * @return string
     */
    public function getProjectVersion()
    {
        if ($this->updateCoreInfo === null) {
			$updateCoreInfo = Model_WebfolioSettings::getSettings('update_core_info');
			
			//Checking for new version...
			$now = time();
			if (($now - $updateCoreInfo['last_checked']) > $this->updateCheckPeriod) {
                $updateCoreInfo = array(
                    'last_checked'=>$now,
                    'version'=>(string)$this->fetchCurrentProjectVersion()
                );

                Model_WebfolioSettings::updateSetting('update_core_info', $updateCoreInfo);
			}
			
			$this->updateCoreInfo = $updateCoreInfo;
        }

        return $this->updateCoreInfo['version'];
    }
	
	/**
     * Retrieves current version from the server on which this 
	 * project is hosted.
     *
     * @return string|null
     */
	protected function fetchCurrentProjectVersion()
	{
		if (in_array('ssl', stream_get_transports())) { //We need openssl extension for this job.
			$zendHttpClient = new Zend_Http_Client();
			$zendHttpClient->setUri($this->projectFilesUri);
			
			$response = $zendHttpClient->request(Zend_Http_Client::GET);
			
			//Extracting version.
			if ($response->isSuccessful() && preg_match('/>\s*WebfolioCMS\-(.+)\.zip\s*</', $response->getBody(), $matches)) {
				return $matches[1];
			}
		}
			
		return null;
	}
}
