<?php
/**
 * Webfolio CMS
 *
 * Copyright (C) 2009 Nikola Posa (http://www.nikolaposa.in.rs)
 *
 * This file is part of Webfolio CMS.
 *
 * Webfolio CMS is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 3 of the License, or 
 * (at your option) any later version.
 *
 * Webfolio CMS is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with Webfolio CMS. If not, see <http://www.gnu.org/licenses/>.
 */
 
/**
 * Controller for managing taxonomy terms.
 *
 * @author Nikola Posa <posa.nikola@gmail.com>
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License
 */
class Taxonomy_AdminController extends Zend_Controller_Action 
{
    const CUSTOM_TAXONOMY_RESOURCE = 'taxonomy_%s';

	/**
     * WebfolioTerms table gateway.
     *
     * @var Taxonomy_Model_DbTable_WebfolioTerms
     */
	protected $webfolioTermsDbTable;
	
	/**
	 * Overloading pre-dispatch hook.
	 *
	 * @return void
	 */
	public function preDispatch()
	{
        $actionName = $this->_request->getActionName();
		//Index action should be automatically routed to add-term action.
		if ($actionName == 'index') {
			$this->_forward('add-term');
		}

        if (!($taxonomy = $this->_request->getParam('taxonomy'))) {
            throw new Exception('Taxonomy param must be supplied in order to use admin controller of taxonomy module.');
        }

        $customResource = sprintf(self::CUSTOM_TAXONOMY_RESOURCE, $taxonomy);
        $privilege = substr($actionName, 0, -5);
        if (WebfolioCMS_Common::getAcl()->has($customResource) && !WebfolioCMS_Common::isUserAllowed(WebfolioCMS_Auth::getInstance()->getIdentity()->group_id, $customResource, $privilege)) {
            throw new Exception('You are not allowed to access requested resource.');
        }
		
		if ((bool)$this->_request->getParam('popup') == true) {
			$this->_helper->layout->setLayoutPath(APPLICATION_PATH . '/modules/admin/layouts/basic');
		}
	}
	
	/**
     * Defined by Zend_Controller_Action.
     *
     * @return void
     */
	public function init()
	{
		$this->webfolioTermsDbTable = new Taxonomy_Model_DbTable_WebfolioTerms();
		
		$this->view->taxonomy = $this->_request->getParam('taxonomy');

		$this->view->assign($this->_helper->termsCustomParams->getTermsParams());
		
		$this->view->popup = (bool)$this->_request->getParam('popup');
	}
	
	/**
     * Handles requests for adding new term.
     *
     * @return void
     */
	public function addTermAction()
	{
		if ($this->_request->isPost()) { //Form submitted?
			$formData = $this->_request->getPost();
			
			$termForm = new Taxonomy_Form_Term();
				
			if ($termForm->isValid($formData)) { //Valid data?	
				$data = $termForm->getValues();
				
				$this->saveTerm($data);
				
				$this->view->message = 'Your term has been successfully saved.';
				$this->view->ok = true;
			}
			else {
				$this->view->fieldValues = $formData;
				$this->view->errorMessages = $termForm->getMessages();
			}
		}
	}
	
	/**
     * Handles requests for editing some term.
     *
     * @return void
     */
	public function editTermAction()
	{
		if ($this->_request->isPost()) { //Form submitted?
			$formData = $this->_request->getPost();
			
			$termForm = new Taxonomy_Form_Term();
				
			if ($termForm->isValid($formData)) { //Valid data?	
				$data = $termForm->getValues();
				
				$this->saveTerm($data);
				
				$this->view->message = 'Your term has been successfully updated.';
				$this->view->ok = true;
			}
			else {
				$this->view->errorMessages = $termForm->getMessages();
			}
			
			$this->view->fieldValues = $formData;
		}
		else {
			$where = $this->webfolioTermsDbTable->getAdapter()->quoteInto('id = ? ', $this->_request->getParam('id'), Zend_Db::INT_TYPE);
			
			$this->view->fieldValues = $this->webfolioTermsDbTable->fetchRow($where, 'parentId');
		}
	}
	
	/**
     * Saves term data.
     *
	 * @param array Data from the term form.
     * @return void
     */
	protected function saveTerm($data)
	{
		$addWhere = array(WebfolioCMS_Filter_TitleUrlFormat::TABLE_ALIAS . '.taxonomy = ?'=>$data['taxonomy']);
		if ($data['id']) {
			$addWhere[WebfolioCMS_Filter_TitleUrlFormat::TABLE_ALIAS . '.id != ?'] = (int)$data['id'];
		}
		$tufFilter = new WebfolioCMS_Filter_TitleUrlFormat('wf_terms', 'name_url_format', $addWhere);
		$data['name_url_format'] = $tufFilter->filter($data['name']);
		
		$parent = null;
		if ($data['parentId'] !== null && $data['parentId'] != $data['currentParentId']) {
			$parent = $data['parentId'];
		}
		
		unset($data['parentId']);
		unset($data['currentParentId']);
		
		if (($id = $data['id'])) { //Edit mode?
			unset($data['id']);
			
			if ($parent !== null) {
				$this->webfolioTermsDbTable->updateNode($data, $id, $parent);
			}
			else {
				$where = $this->webfolioTermsDbTable->getAdapter()->quoteInto('id = ?', $id, Zend_Db::INT_TYPE);
				$this->webfolioTermsDbTable->update($data, $where);
			}
		}
		else {
			$this->webfolioTermsDbTable->insert($data, $parent);
		}
		
		//Cleaning any taxonomy-related data in cache.
		Zend_Registry::get('CacheManager')->getCache('default')->clean(Zend_Cache::CLEANING_MODE_MATCHING_TAG, array('taxonomy_' . $data['taxonomy']));
	}
}
?>