<?php
/**
 * Webfolio CMS
 *
 * Copyright (C) 2009 Nikola Posa (http://www.nikolaposa.in.rs)
 *
 * This file is part of Webfolio CMS.
 *
 * Webfolio CMS is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 3 of the License, or 
 * (at your option) any later version.
 *
 * Webfolio CMS is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with Webfolio CMS. If not, see <http://www.gnu.org/licenses/>.
 */
 
/**
 * Main Taxonomy module model. 
 *
 * @author Nikola Posa <posa.nikola@gmail.com>
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License
 */
class Taxonomy_Model_Main extends Model_WebfolioBase
{
	const MULTI_TERMS_SEPARATOR = ',';
	
	/**
	 * Name of the taxonomy.
	 *
	 * @var string
	 */
	protected $taxonomy;
	
	/**
	 * WebfolioTerms table data gateway.
	 *
	 * @var Zend_Db_Table
	 */
	protected $termsDbTable;
	
	/**
	 * WebfolioTermRelations table data gateway.
	 *
	 * @var Zend_Db_Table
	 */
	protected $termRelationsDbTable;
	
	/**
	 * Constructor.
	 *
	 * @param string Name of the taxonomy.
	 * @return Taxonomy_Model_Main
	 */
	public function __construct($taxonomy)
	{
		parent::__construct();
		
		$this->taxonomy = (string)$taxonomy;
		
		$this->termsDbTable = new Taxonomy_Model_DbTable_WebfolioTerms();
		
		$this->termRelationsDbTable = new Taxonomy_Model_DbTable_WebfolioTermRelations();
	}
	
	/**
	 * Retrieves WebfolioTerms table data gateway instance.
	 *
	 * @return Zend_Db_Table
	 */
	public function getWebfolioTermsDbTable()
	{
		return $this->termsDbTable;
	}
	
	/**
	 * Retrieves WebfolioTermRelations table data gateway instance.
	 *
	 * @return Zend_Db_Table
	 */
	public function getWebfolioTermRelationsDbTable()
	{
		return $this->termRelationsDbTable;
	}
	
	/**
	 * Checks whether taxonomy module is enabled and active.
	 *
	 * @return bool
	 */
	public static function isEnabled()
	{
		if (in_array('taxonomy', Model_WebfolioSettings::getSettings('active_modules'))) {
			return true;
		}
		
		return false;
	}
	
	/**
	 * DEPRECATED! Use TermsCustomParams view helper instead.
	 * 
	 * Stores user supplied options for customizing terms grid and 
	 * form, in session, so they can be handled in appropriate 
	 * place. This should be called by modules which use taxonomy 
	 * module, usually in init() method of action controller on 
	 * which taxonomy helpers are used.
	 *
	 * Supported params for $options are:
     *   keyword - Word that describes your taxonomy and will be used in many rendering situations (form heading title, grid empty message, etc.),
     *   nameColLabel - Header lable of form term name column in terms grid (string),
	 *   termNameLabel - Label of term name field in form for adding/editing term (string),
	 * 	 useParent - Whether to enable selection of parent term in form for adding/editing term (bool)
	 *	 parentTermLabel - Label of combo box for selecting parent term.
	 *
	 * @param array Options for generating terms grid and form.
	 * @return void
	 */
	public static function setTermsCustomParams($options)
	{
		$WFTaxonomySession = new Zend_Session_Namespace('WFTaxonomy');
		$termsCustomParams = $WFTaxonomySession->__get('termsCustomParams');
		
		$WFTaxonomySession->termsCustomParams = array_merge($termsCustomParams, $options);
	}
	
	/**
	 * Retrieves terms data related to some content.
	 *
	 * @param int Id of some webfolio content which uses taxonomy.
	 * @return array
	 */
	public function getTerms($contentId)
	{
		$contentId = (int)$contentId;
		
		$sql = "SELECT t.* 
		FROM " . $this->termsDbTable->info('name') . " AS t
		INNER JOIN " . $this->termRelationsDbTable->info('name') . " AS tr ON t.id = tr.term_id
		WHERE t.taxonomy = " . $this->dbHandler->quote($this->taxonomy) . " AND tr.content_id = " . $this->dbHandler->quote($contentId, Zend_Db::INT_TYPE);
		
		return $this->dbHandler->fetchAll($sql);
	}
	
	/**
	 * Retrieves terms tree for supplied taxonomy.
	 *
	 * @param array|string|null Additional SQL where clause. (optional)
	 * @return array
	 */
	public function getTermsTree($addWhere = null)
	{
		$where = array(Taxonomy_Model_DbTable_WebfolioTerms::LEFT_TBL_ALIAS . '.taxonomy = ?' => $this->taxonomy);
		
		if ($addWhere != null && (is_array($addWhere) || is_string($addWhere))) {	
			$addWhere = (array) $addWhere;
			
			$where = array_merge($addWhere, $where);
		}
		
		return $this->termsDbTable->getTree($where);
	}
	
	/**
	 * Saves some term-content relation.
	 *
	 * @param int Id of some webfolio content. 
	 * @param int|array Id of new term or array of term names. 
	 * @return void
	 */
	public function saveTermRelations($contentId, $term)
	{
		$taxonomy = $this->dbHandler->quote($this->taxonomy);
		
		if (is_numeric($term)) { //Combo box?
			$sql = 'SELECT term_id FROM wf_term_relations WHERE content_id = ' . $this->dbHandler->quote($contentId, Zend_Db::INT_TYPE) . ' AND term_id IN (SELECT id FROM wf_terms WHERE taxonomy = ' . $taxonomy . ')';
			$result = $this->dbHandler->fetchRow($sql);
			
			if ($result) {
				$currentTermId = $result['term_id'];
				$where = 'content_id = ' . $this->dbHandler->quote($contentId, Zend_Db::INT_TYPE) . ' AND term_id = ' . $currentTermId;
				
				$this->termRelationsDbTable->update(array('term_id'=>$term), $where);
			}
			else {
				$this->termRelationsDbTable->insert(array('content_id'=>$contentId, 'term_id'=>$term));
			}
		}
		elseif (is_array($term)) { //Multi-terms (textfield)?
			//We should first remove all previous term relations.
			$this->removeTermRelations($contentId);
			
			$termIds = array();
			foreach ($term as $t) {	
				$t = trim($t);
				
				$termId = null;
				
				$where = $this->termsDbTable->getAdapter()->quoteInto("taxonomy = '$this->taxonomy' AND name = ?", $t);
				
				if (!($term = $this->termsDbTable->fetchRow($where))) { //If term with supplied name is not in database, we should insert it.
					$tufFilter = new WebfolioCMS_Filter_TitleUrlFormat('wf_terms', 'name_url_format', array(WebfolioCMS_Filter_TitleUrlFormat::TABLE_ALIAS . '.taxonomy = ?'=>$this->taxonomy));
					$data = array('name'=>$t, 'name_url_format'=>$tufFilter->filter($t), 'taxonomy'=>$this->taxonomy);
					$termId = $this->termsDbTable->insert($data);
					
					$this->termRelationsDbTable->insert(array('content_id'=>$contentId, 'term_id'=>$termId));
				}
				elseif (!in_array($term['id'], $termIds)) { //Same relationship not already inserted?
					$termIds[] = $term['id'];
					
					$this->termRelationsDbTable->insert(array('content_id'=>$contentId, 'term_id'=>$term['id']));
				}		
			}
			
			//Removing tags that not related to any content. This may occur 
			//after user removed some tags from some content and those tags 
			//aren't related to any other content.
			$this->termsDbTable->delete('taxonomy = ' . $this->termsDbTable->getAdapter()->quote($this->taxonomy) . ' AND id NOT IN (SELECT DISTINCT term_id FROM ' . $this->termRelationsDbTable->info('name') . ')');
			
			//Tags are added, so we should clear taxonomy-related cache.
			Zend_Registry::get('CacheManager')->getCache('default')->clean(Zend_Cache::CLEANING_MODE_MATCHING_TAG, array('taxonomy_' . $this->taxonomy));
		}
	}
	
	/**
	 * Removes relations for some webfolio content.
	 *
	 * @param int Id of some webfolio content. 
	 * @return void
	 */
	public function removeTermRelations($contentId)
	{
		$taxonomy = $this->dbHandler->quote($this->taxonomy);
		
		$where = 'content_id = ' . $this->dbHandler->quote($contentId, Zend_Db::INT_TYPE) . ' AND term_id IN (SELECT id FROM wf_terms WHERE taxonomy = ' . $taxonomy . ')';
		$this->termRelationsDbTable->delete($where);
	}
}
?>