<?php
/**
 * Webfolio CMS
 *
 * Copyright (C) 2009 Nikola Posa (http://www.nikolaposa.in.rs)
 *
 * This file is part of Webfolio CMS.
 *
 * Webfolio CMS is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 3 of the License, or 
 * (at your option) any later version.
 *
 * Webfolio CMS is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with Webfolio CMS. If not, see <http://www.gnu.org/licenses/>.
 */
 
/**
 * Hook that holds registered Webfolio event handlers and it 
 * is used as broker for calling appropriate event hooks.
 *
 * @author Nikola Posa <posa.nikola@gmail.com>
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License
 */
class WebfolioCMS_Events_Broker extends WebfolioCMS_HookContainer_Abstract
{
	const WF_COMMENT_PRE_SAVE 	   = 'WFCommentPreSave';
	const WF_COMMENT_POST_SAVE 	   = 'WFCommentPostSave';
	const WF_ENTRY_FORM_POPULATE   = 'WFEntryFormPopulate';
	const WF_ENTRY_FORM_PRE_RENDER = 'WFEntryFormPreRender';
	const WF_ENTRY_FORM_PRE_VALID  = 'WFEntryFormPreValidation';
	const WF_ENTRY_SAVE 		   = 'WFEntrySave';
	const WF_ENTRY_DELETE 		   = 'WFEntryDelete';

	/**
     * Valid events.
     *
     * @var array
     */
	protected $_validEvents = array(
		self::WF_COMMENT_PRE_SAVE,
		self::WF_COMMENT_POST_SAVE,
		self::WF_ENTRY_FORM_POPULATE,
		self::WF_ENTRY_FORM_PRE_RENDER,
        self::WF_ENTRY_FORM_PRE_VALID,
        self::WF_ENTRY_SAVE,
        self::WF_ENTRY_DELETE
	);

	/**
	 * Singleton instance.
	 *
	 * @var WebfolioCMS_Events_Broker
	 */
    protected static $_instance = null;

    /**
	 * Event handlers, which this container is holding, are grouped by
     * event name, which is passed in add() method when registering
     * handler.
	 *
	 * @var string|null
	 */
    protected $groupingIndex = 'event';

    /**
	 * Simulates cache for handler object that were created during some
     * event handling. For example, one module can register multiple
     * event handlers on some methods which are in same class, so we
     * should make sure that object of that class is created only once.
	 *
	 * @var array
	 */
    protected $handlerObjects = array();
	
	/**
	 * Defined by WebfolioCMS_Singleton_Interface.
     *
	 * Singleton pattern implementation.
	 *
	 * @return WebfolioCMS_Events_Broker
	 */
	public static function getInstance()
	{
		if (null === self::$_instance) {
            self::$_instance = new self();
        }

        return self::$_instance;
	}
	
	/**
	 * Defined by WebfolioCMS_HookContainer_Interface. 
	 *
	 * Retrieves registered WFEntry plugins.
	 *
	 * @return array
	 */
	public static function get()
	{
		$instance = self::getInstance();
		
		return $instance->data;
	}
	
	/**
	 * Defined by WebfolioCMS_HookContainer_Interface. 
	 *
	 * Adds event handler onto stack.
	 *
	 * @param array Event handler data.
	 * @return void
	 */
	public static function add($data)
	{
		$instance = self::getInstance();

        if (array_key_exists($instance->groupingIndex, $data)
            && in_array($data[$instance->groupingIndex], $instance->_validEvents)
            && array_key_exists('handler', $data)
            && is_callable($data['handler'])
            && is_array($data['handler']) && is_string($data['handler'][0])
        ) {
            $instance->store($data);
        }
	}
	
	/**
	 * Notifies registered event handlers of some event. 
	 *
	 * @param string Event name.
	 * @param array Additional params that will be passed to event handler method (optional).
	 * @return void
	 */
	public static function notifyEventHandlers($event, array $params = array())
	{
		$instance = self::getInstance();

        if (array_key_exists($event, $instance->data)) {
            foreach ($instance->data[$event] as $item) {
                $className = $item['handler'][0];
                $methodName = $item['handler'][1];
                
                if (!array_key_exists($className, $instance->handlerObjects)) {
                    $instance->handlerObjects[$className] = new $className();
                }

                call_user_func_array(array($instance->handlerObjects[$className], $methodName), $params);
            }
        }
	}
}
?>