<?php
/**
 * Webfolio CMS
 *
 * Copyright (C) 2009 Nikola Posa (http://www.nikolaposa.in.rs)
 *
 * This file is part of Webfolio CMS.
 *
 * Webfolio CMS is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 3 of the License, or 
 * (at your option) any later version.
 *
 * Webfolio CMS is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with Webfolio CMS. If not, see <http://www.gnu.org/licenses/>.
 */
 
/**
 * Filter for extracting only small subset of some text, based 
 * on supplied length, removing all HTML markup from it, and 
 * making sure that text isn't just roughly breaked. 
 * This filter is directly utilized by getShortDesc() method 
 * of WebfolioEntries model.
 *
 * @author Nikola Posa <posa.nikola@gmail.com>
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License
 */
class WebfolioCMS_Filter_Teaser implements Zend_Filter_Interface
{
	/**
	 * Desired length of filtered text expressed in number of 
	 * characters.
	 *
	 * @var int
	 */
	protected $length = 100;
	
	/**
	 * Character which will be used as a delimiter when limiting 
	 * and shortening supplied text.
	 *
	 * @var string
	 */
	protected $delimiter = '.';
	
	/**
	 * Some string to be added at the end of filtered text, usually 
	 * "...".
	 *
	 * @var string
	 */
	protected $suffix = '';
	
	/**
     * Constructor.
     *
     * @param array Options for configuring this filter.
     * @return void
     */
    public function __construct($options = array())
    {
        $this->setOptions($options);
    }
	
	/**
     * Sets options for configuring this filter.
     *
     * @param array Options for configuring this filter.
     * @return void
     */
	public function setOptions($options)
	{
		if (array_key_exists('length', $options)) {
            $this->setLength($options['length']);
        }

        if (array_key_exists('delimiter', $options)) {
            $this->setDelimiter($options['delimiter']);
        }
		
		if (array_key_exists('suffix', $options)) {
            $this->setSuffix($options['suffix']);
        }
	}
	
	/**
     * Returns the "length" option.
     *
     * @return int
     */
    public function getLength()
    {
        return $this->length;
    }

    /**
     * Sets the "length" option.
     *
     * @param int Length of filtered text expressed in number of characters
     * @return WebfolioCMS_Filter_Teaser
     */
    public function setLength($length)
    {
       $this->length = (int) $length;
       return $this;
    }
	
	/**
     * Returns the "delimiter" option.
     *
     * @return string
     */
    public function getDelimiter()
    {
        return $this->delimiter;
    }

    /**
     * Sets the "delimiter" option.
     *
     * @param string Character which will be used as a delimiter when shortening supplied text.
     * @return WebfolioCMS_Filter_Teaser
     */
    public function setDelimiter($delimiter)
    {
       $this->delimiter = (string) $delimiter;
       return $this;
    }
	
	/**
     * Returns the "suffix" option.
     *
     * @return string
     */
    public function getSuffix()
    {
        return $this->suffix;
    }

    /**
     * Sets the "suffix" option.
     *
     * @param string Some string to be added at the end of filtered text, usually "...".
     * @return WebfolioCMS_Filter_Teaser
     */
    public function setSuffix($suffix)
    {
       $this->suffix = (string) $suffix;
       return $this;
    }
	
	/**
	 * Defined by Zend_Filter_Interface.
	 *
	 * @param string Some text.
	 * @return string
	 */
	public function filter($value)
    {
		$value = strip_tags($value);
		
		if (strlen($value) > $this->length) {
			$offset = (($pos = strpos($value, $this->delimiter, $this->length)) !== false) ? $pos : $this->length;
			
			$value = substr($value, 0, $offset);
		}
		
		return $value . $this->suffix;
    }
}
?>