<?php
/**
 * Webfolio CMS
 *
 * Copyright (C) 2009 Nikola Posa (http://www.nikolaposa.in.rs)
 *
 * This file is part of Webfolio CMS.
 *
 * Webfolio CMS is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 3 of the License, or 
 * (at your option) any later version.
 *
 * Webfolio CMS is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with Webfolio CMS. If not, see <http://www.gnu.org/licenses/>.
 */
 
/**
 * Filter for generating SEO friendly title. Two additional 
 * options can be passed to this filter - db table name and 
 * name of the title_url_format-like column in that table, 
 * and those will be used to check whether same title url 
 * format already exists in some db table, as usually value of 
 * that column should be unique. In case that there is such 
 * colision, this filter will generate unique title url format, 
 * by prepending numbers to it, or example: foo1, foo2, etc.
 *
 * @author Nikola Posa <posa.nikola@gmail.com>
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License
 */
class WebfolioCMS_Filter_TitleUrlFormat extends WebfolioCMS_Filter_ConvertAccents
{
	const TUF_COLUMN = 'wf_filter_tuf';
	
	const TABLE_ALIAS = 't1';
	
	/**
     * Database adapter that will be used in case additional db 
	 * table collision check should be performed.
     *
     * @var Zend_Db_Adapter_Abstract
     */
	protected $_dbHandler;
	
	/**
     * Name of the database table which has title_url_format-like 
	 * column.
     *
     * @var string
     */
	protected $_dbTableName = null;
	
	/**
     * Name title_url_format-like column.
     *
     * @var string
     */
	protected $_titleUrlFormatCol = null;
	
	/**
     * Additional where clause that will be used in query that 
	 * checks for collision.
     *
     * @var mixed
     */
	protected $_addWhere = null;
	
	/**
     * Constructor.
     *
     * @param string|null Name of the database table which has title_url_format-like column. (optional)
	 * @param string|null Name title_url_format-like column. (optional)
	 * @param string|array Additional where clause that will be used in query that checks for collision. (optional)
     * @return void
     */
    public function __construct($dbTableName = null, $titleUrlFormatCol = null, $addWhere = null)
    {
        if ($dbTableName != null && $titleUrlFormatCol != null) {
			$this->setDbTableName($dbTableName);
			$this->setTitleUrlFormatCol($titleUrlFormatCol);
			
			if ($addWhere != null) {
				$this->setAddWhere($addWhere);
			}
		}
    }
	
	/**
     * Sets database adapter.
     *
     * @param Zend_Db_Adapter_Abstract Database adapter.
     * @return WebfolioCMS_Filter_TitleUrlFormat
     */
	public function setDbHandler(Zend_Db_Adapter_Abstract $dbHandler)
	{
		$this->_dbHandler = $dbHandler;
		return $this;
	}
	
	/**
     * Gets database adapter. Default adapter will be one retrieved 
	 * from registry.
     *
     * @return Zend_Db_Adapter_Abstract
     */
	public function getDbHandler()
	{
		if ($this->_dbHandler == null) {
			$this->_dbHandler = Zend_Registry::get('dbHandler');
		}
		
		return $this->_dbHandler;
	}
	
	/**
     * Sets name of the database table.
     *
     * @param string Name of the database table.
     * @return WebfolioCMS_Filter_TitleUrlFormat
     */
	public function setDbTableName($dbTableName)
	{
		$this->_dbTableName = (string)$dbTableName;
		return $this;
	}
	
	/**
     * Gets name of the database table.
     *
     * @return string
     */
	public function getDbTableName()
	{
		return $this->_dbTableName;
	}
	
	/**
     * Sets name title_url_format-like column.
     *
     * @param string Name title_url_format-like column.
     * @return WebfolioCMS_Filter_TitleUrlFormat
     */
	public function setTitleUrlFormatCol($titleUrlFormatCol)
	{
		$this->_titleUrlFormatCol = (string)$titleUrlFormatCol;
		return $this;
	}
	
	/**
     * Gets name title_url_format-like column.
     *
     * @return string
     */
	public function getTitleUrlFormatCol()
	{
		return $this->_titleUrlFormatCol;
	}
	
	/**
     * Sets additional where clause.
     *
     * @param string|array Additional where clause.
     * @return WebfolioCMS_Filter_TitleUrlFormat
     */
	public function setAddWhere($addWhere)
	{
		if ($addWhere != null && (is_string($addWhere) || is_array($addWhere))) {
			$this->_addWhere = $addWhere;
		}
		
		return $this;
	}
	
	/**
     * Gets additional where clause.
     *
     * @return string
     */
	public function getAddWhere()
	{
		return $this->_addWhere;
	}
	
	/**
	 * Defined by Zend_Filter_Interface.
	 *
	 * Returns the string $value, converting characters to lowercase, 
	 * replacing spaces and underscores with dash (-),  and removing 
	 * all but alphabetic and digit characters.
	 *
	 * @param string Some string (i.e.: webfolio item title).
	 * @return string
	 */
	public function filter($value)
    {
		$titleUrlFormat = parent::filter($value);
		
		//Lowercase characters.
        $titleUrlFormat = strtolower($titleUrlFormat);
		
		//Replacing non alfa-numeric characters with dash ('-').
		$titleUrlFormat = preg_replace('/[^a-zA-Z0-9]/', '-', $titleUrlFormat);
		
		//Triming dashes.
		$titleUrlFormat = trim($titleUrlFormat, '-');
		
		//Making sure that there's no more that one dash character in a row.
		if (strpos($titleUrlFormat, '--') !== false) {
			$titleUrlFormat = preg_replace('/\-{2,}/', '-', $titleUrlFormat);
		}
		
		if ($this->_dbTableName != null) { //Additional db table check should be executed?
			$titleUrlFormat = $this->checkForCollision($titleUrlFormat);
		}
		
		return $titleUrlFormat;
    }
	
	/**
	 * Performs checking whether same title url format already exists 
	 * in supplied $_dbTableName, as usually value of that 
	 * $_titleUrlFormatCol column should be unique. In case that there 
	 * is such colision, this method will generate unique title url 
	 * format, by prepending numbers to it, or example: foo1, foo2, 
	 * etc.
	 *
	 * @param string Generated title url format.
	 * @return string
	 */
	protected function checkForCollision($titleUrlFormat)
	{
		$dbHandler = $this->getDbHandler();
			
		//Building query.
		$select = $dbHandler->select();
		$select = $select->from(array(self::TABLE_ALIAS => $this->_dbTableName), array(self::TABLE_ALIAS . '.*'))
			->join(array('t2' => $this->_dbTableName), '', array(self::TUF_COLUMN=>'t2.' . $this->_titleUrlFormatCol))
			->where(self::TABLE_ALIAS . '.' . $this->_titleUrlFormatCol . ' = ?', $titleUrlFormat)
			->where('t2.' . $this->_titleUrlFormatCol . ' REGEXP ' . $dbHandler->quote('^' . $titleUrlFormat . '[0-9]*$'));
		
		if ($this->_addWhere != null) { //Has some additional where clause?
			$addWhere = (array) $this->_addWhere;

			foreach ($addWhere as $key=>$val) {
				if (is_int($key)) {
					$select->where($val);
				} else {
					$select->where($key, $val);
				}
			}
		}
			
		if (count($result = $dbHandler->fetchAll($select)) > 0) { //There is some collision.
			$numbers = array();
				
			foreach ($result as $row) {
				$numbers[] = (int)substr($row[self::TUF_COLUMN], strlen($titleUrlFormat));
			}
			
			sort($numbers);
			
			$last = $numbers[count($numbers)-1];
			if (count($diff = array_diff(range($numbers[0], $last), $numbers)) > 0) { //Not all numbers in scope are used? Then get first unused. 
				$titleUrlFormat .= current($diff);
			}
			else {
				$titleUrlFormat .= ($last+1);
			}
		}
		
		return $titleUrlFormat;
	}
}
?>