<?php
/**
 * Webfolio CMS
 *
 * Copyright (C) 2009 Nikola Posa (http://www.nikolaposa.in.rs)
 *
 * This file is part of Webfolio CMS.
 *
 * Webfolio CMS is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 3 of the License, or 
 * (at your option) any later version.
 *
 * Webfolio CMS is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with Webfolio CMS. If not, see <http://www.gnu.org/licenses/>.
 */
 
/**
 * Base class for all hook handlers. Utilizes Singleton 
 * pattern to persist data over the requests, so modules can 
 * use hooks to contribute some functionality to Webfolio 
 * CMS.
 *
 * @author Nikola Posa <posa.nikola@gmail.com>
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License
 */
abstract class WebfolioCMS_HookContainer_Abstract implements WebfolioCMS_HookContainer_Interface
{
	/**
	 * Array that will be used to persist hook data, after 
	 * classes that extend this class implement singleton pattern.
	 *
	 * @var array
	 */
    protected $data = array();
	
	/**
	 * Index of hook data array by which all array will be 
	 * grouped.
	 *
	 * @var string|null
	 */
    protected $groupingIndex = null;
	
	/**
	 * Constructor.
	 *
	 * @return WebfolioCMS_HookContainer_Abstract
	 */
	protected function __construct()
	{
	}
	
	/**
	 * Does the adding of hook data onto stack.
	 *
	 * @param array Hook data.
	 * @return void
	 */
	protected function _doAdd($hookData)
	{
		if ($this->isValid($hookData)) {
			$hookData = $this->prepare($hookData);
			
			$this->store($hookData);
		}
	}
	
	/**
	 * Runs some basic validation in case $groupingIndex is set, so
     * that key must be in $hookData. Extending classes should
     * override this method in order to implement some custom
     * validation.
	 *
	 * @param array Hook data.
	 * @return bool
	 */
	 protected function isValid($hookData)
	 {
		if ($this->groupingIndex == null || ($this->groupingIndex != null && array_key_exists($this->groupingIndex, $hookData))) {
			return true;
		}
		
		return false;
	 }
	
	/**
	 * Does some preparation of hook data. Called before adding hook 
	 * data onto stack in _doAdd() method.
	 *
	 * @param array Hook data.
	 * @return array
	 */
	 protected function prepare($hookData)
	 {
		return $hookData;
	 }
	 
	 /**
	 * Stores hook data onto main stack of data.
	 *
	 * @param array Hook data.
	 * @return void
	 */
	protected function store($hookData)
	{
		if ($this->groupingIndex !== null) { 
			$groupingIndex = $hookData[$this->groupingIndex];
			unset($hookData[$this->groupingIndex]);
            
            if (empty($this->data) || !isset($this->data[$groupingIndex]) || (isset($this->data[$groupingIndex]) && !in_array($hookData, $this->data[$groupingIndex]))) { //Preventing duplicates.
                $this->data[$groupingIndex][] = $hookData;
            }
		}
		elseif (empty($this->data) || (is_numeric(key($this->data)) && !in_array($hookData, $this->data))) { //Preventing duplicates.
			$this->data[] = $hookData;
		}
	}
}
?>