<?php
/**
 * Webfolio CMS
 *
 * Copyright (C) 2009 Nikola Posa (http://www.nikolaposa.in.rs)
 *
 * This file is part of Webfolio CMS.
 *
 * Webfolio CMS is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 3 of the License, or 
 * (at your option) any later version.
 *
 * Webfolio CMS is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with Webfolio CMS. If not, see <http://www.gnu.org/licenses/>.
 */
 
/**
 * Class which provides some function for manipulating nested set 
 * data.
 *
 * @author Nikola Posa <posa.nikola@gmail.com>
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License
 */
class WebfolioCMS_NestedSetManipulation
{
	/**
	 * Key that holds depth information, in array that will be 
	 * supplied to functions of this class.
	 *
	 * @var string
	 */
	protected $depthKey = 'depth';
	
	/**
	 * Constructor.
	 *
	 * @param string|null Key that holds depth information. (optional, "depth" by default)
	 * @return void
	 */
	public function __construct($depthKey = null)
	{
		if ($depthKey != null) {
			$this->depthKey = (string)$depthKey;
		}
	}
	
	/**
	 * Adds additional element to every nested row in $items, which 
	 * will hold its parent elements.
	 *
	 * @param array Array fetched from nested set database table.
	 * @param string String that will be used as key for parent items. (optional)
	 * @param string|null If this param is supplied, parents array will consist only of values from $items array with keys with value of this param. (optional)
	 * @return array
	 */
	public function addParentsInfo($items, $parentKey = 'parents', $specific = null)
	{
		$depth0 = -1;
		
		foreach ($items as $key=>$item) {
			$depth = (int)$item[$this->depthKey];
			
			if ($depth != 0) {
				$parents = array();
				$temp = array_slice($items, $depth0, $key-$depth0+1);
				foreach ($temp as $i=>$val) {
					$d = (int)$val[$this->depthKey];
					if ($d < $depth && (int)$temp[$i+1][$this->depthKey] != $d) { //Making sure that last node on same level is retrieved as parent.
						$parents[] = ($specific != null && array_key_exists($specific, $val)) ? $val[$specific] : $val;
					}
				}
				
				$items[$key][$parentKey] = $parents;
			}
			else {
				$depth0 = $key;
			}
		}
		
		return $items;
	}
	
	/**
	 * Generate nested array from the array fetched from some nested 
	 * set database table.
	 *
	 * @param array Array fetched from nested set database table.
	 * @param string String that will be used as key of a array that holds nested items. (optional)
	 * @return array
	 */
	public function nestedToArray($items, $groupBy = null)  
	{  
		$tree = array();  
		$depths = array();  
			
		foreach ($items as $key=>$item) { 
			$depth = (int)$item[$this->depthKey];
			unset($item[$this->depthKey]);
			
			if ($depth == 0) {
				$tree[$key] = $item;  
			}  
			else {  
				$parent = &$tree;  
				
				for ($i = 1; $i <= $depth; $i++) {  	
					if ($groupBy != null) {
						$parent = &$parent[$depths[$i]][(string)$groupBy];
					}
					else {
						$parent = &$parent[$depths[$i]];  
					}
				} 
				   
				$parent[$key] = $item;
			}
			
			$depths[$depth + 1] = $key;  
		}  
		  
		return $tree;  
	}  
}

