<?php
/**
 * Webfolio CMS
 *
 * Copyright (C) 2009 Nikola Posa (http://www.nikolaposa.in.rs)
 *
 * This file is part of Webfolio CMS.
 *
 * Webfolio CMS is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 3 of the License, or 
 * (at your option) any later version.
 *
 * Webfolio CMS is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with Webfolio CMS. If not, see <http://www.gnu.org/licenses/>.
 */
 
/**
 * Custom paginator adapter, for paginating data which comes 
 * from some model.
 *
 * @author Nikola Posa <posa.nikola@gmail.com>
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License
 */
class WebfolioCMS_Paginator_Adapter_Model implements Zend_Paginator_Adapter_Interface
{
    const ROW_COUNT_COLUMN = 'wf_paginator_row_count';

    /**
     * Model_Base instance.
     *
     * @var Zend_Db_Select
     */
    protected $_model;

	/**
     * Name of the $model method that will be used for retrieveing 
	 * items.
     *
     * @var string
     */
    protected $_methodName = 'fetchAll';
	
    /**
     * Optional where clause that is used when fetching data.
     *
     * @var mixed
     */
    protected $_where = null;
	
	/**
     * Optional order clause that is used when fetching data.
     *
     * @var mixed
     */
    protected $_order = null;

    /**
     * Total rows count.
     *
     * @var int
     */
    protected $_rowCount;

    /**
     * Constructor.
     *
	 * Supported params for $options:
	 *   methodName - optional Name of the $model method that will be used for retrieveing items, which has the same definition as default fetchAll() method (string).
	 *   where - optional SQL WHERE clause that will be passed to "methodName" (string|array).
	 *   order - optional SQL ORDER clause that will be passed to "methodName" (string|array).
     * @param Model_Base Model_Base instance.
	 * @param array Options for using this adapter as key-value pairs.
     */
    public function __construct(Model_Base $model, $options = array())
    {
        $this->_model = $model;
		
		foreach ($options as $key=>$value) {
			switch ($key) {
				case 'methodName':
					$this->_methodName = (string)$value;
					break;
				case 'where': case 'order':
					if (is_array($value) || is_string($value)) {
						$key = '_' . $key;
						$this->$key = $value;
					}
					break;
				default: 
					break;
			}
		}
    }

	/**
     * Sets the total row count directly.
     *
     * @param int $totalRowCount Total row count integer.
     * @return WebfolioCMS_Paginator_Adapter_Model
     */
    public function setRowCount($rowCount)
    {
        if (is_integer($rowCount)) {
            $this->_rowCount = $rowCount;
        } 
		
        return $this;
    }
	
    /**
     * Calculates and returns total rows count, in case it wasn't 
	 * already set. This function uses Zend_Db_Table instance, that 
	 * is used as gateway for supplied model, to generate count 
	 * query that will calculate total number of rows for supplied 
	 * data.
	 *
     * @return int
     */
    public function getRowCount()
    {
		if ($this->_rowCount === null) {
			$dbTable = $this->_model->getMapper()->getDbTable();
		   
			$countSelect = $dbTable->select()->from($dbTable->info(Zend_Db_Table_Abstract::NAME), array(self::ROW_COUNT_COLUMN => new Zend_Db_Expr('COUNT(1)')));
		   
			if (!is_array($this->_where)) {
				$this->_where = array($this->_where);
			}
			
			foreach ($this->_where as $key => $val) {
				if (is_int($key)) {
					$countSelect->where($val);
				} else {
					$countSelect->where($key, $val);
				}
			}
			
			if (!is_array($this->_order)) {
				$this->_order = array($this->_order);
			}
			
			foreach ($this->_order as $val) {
				$countSelect->order($val);
			}
			
			$result = $countSelect->query(Zend_Db::FETCH_ASSOC)->fetch();
			
			$this->_rowCount = count($result) > 0 ? (int)$result[self::ROW_COUNT_COLUMN] : 0;
		}
		
		return $this->_rowCount;
    }

    /**
     * Defined by Zend_Paginator_Adapter_Interface.
     *
     * @param  integer $offset Page offset
     * @param  integer $itemCountPerPage Number of items per page
     * @return array
     */
    public function getItems($offset, $itemCountPerPage)
    {
		return call_user_func_array(array($this->_model, $this->_methodName), array($this->_where, $this->_order, $itemCountPerPage, $offset));
    }

    /**
     * Defined by Countable.
     *
     * @return int
     */
    public function count()
    {
		return $this->getRowCount();
    }
}
