<?php
/**
 * Webfolio CMS
 *
 * Copyright (C) 2009 Nikola Posa (http://www.nikolaposa.in.rs)
 *
 * This file is part of Webfolio CMS.
 *
 * Webfolio CMS is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 3 of the License, or 
 * (at your option) any later version.
 *
 * Webfolio CMS is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with Webfolio CMS. If not, see <http://www.gnu.org/licenses/>.
 */
 
/**
 * Abstract ViewHelper for generating head section. This 
 * helper utilizes ZF view helpers for generating head 
 * section elements.
 *
 * @author Nikola Posa, www.nikolaposa.in.rs
 * @license http://opensource.org/licenses/gpl-3.0.html GNU General Public License
 */
abstract class WebfolioCMS_View_Helper_HeadSectionAbstract extends Zend_View_Helper_Abstract 
{
	const FILE = 'FILE';
	const SOURCE = 'SOURCE';
	
	/**
	 * Whether to output content-type meta tag.
	 *
	 * @var bool
	 */
	protected $appendContentType = false;
	
	/**
	 * Indentation of head section elements (measured in 
	 * number of space characters).
	 *
	 * @var int
	 */
	protected $indent = 8;
	
	/**
	 * Name of requested module.
	 *
	 * @var string
	 */
	protected $module;
	
	/**
	 * Name of requested controller.
	 *
	 * @var string
	 */
	protected $controller;
	
	/**
	 * Name of requested action.
	 *
	 * @var string
	 */
	protected $action;
	
	/**
	 * Constructor.
	 *
	 * @return WebfolioCMS_View_Helper_HeadSectionAbstract
	 */
	public function __construct() 
	{
		$this->module = Zend_Controller_Front::getInstance()->getRequest()->getModuleName();
		
		$this->controller = Zend_Controller_Front::getInstance()->getRequest()->getControllerName();
		
		$this->action = Zend_Controller_Front::getInstance()->getRequest()->getActionName();
	}
	
	/**
	 * Sets $appendContentType flag.
	 *
	 * @param bool True or false
	 * @return void
	 */
	public function setAppendContentType($flag)
	{
		$this->appendContentType = $flag;
	}
	
	/**
	 * Sets indent.
	 *
	 * @param int Indent (number of space characters).
	 * @return void
	 */
	public function setIndent($indent)
	{
		$this->indent = (int)$indent;
	}
	
	/**
	 * Adds title.
	 *
	 * @param string Some title.
	 * @param string Placement (append, prepend, set).
	 * @return void
	 */
	public function addTitle($title, $placement = 'APPEND')
	{
		$this->view->headTitle($title, $placement)->setAutoEscape(false);
	}
	
	/**
	 * Adds CSS stylesheet.
	 *
	 * @param string Value.
	 * @param string Mode (file or source).
	 * @param string Placement (append, prepend, set).
	 * @return void
	 */
	public function addCss($css, $mode = 'FILE', $placement = 'APPEND')
	{
		if (is_array($css)) {
			if ($placement == 'PREPEND') {
				$css = array_reverse($css);
			}
			foreach ($css as $value) {
				$this->addCss($value, $mode, $placement);
			}
		}
		elseif (is_string($css)) {
			if ($mode == 'FILE') {
				$methodName = strtolower($placement) . 'Stylesheet';
				
				$this->view->headLink()->$methodName($css);
			}
			elseif ($mode == 'SOURCE') {
				$this->view->headStyle($css, $placement);
			}
		}
	}
	
	/**
	 * Adds JavaScript (path to file or source).
	 *
	 * @param string Value.
	 * @param string Mode (file or source).
	 * @param string Placement (append, prepend, set).
	 * @return void
	 */
	public function addJs($script, $mode = 'FILE', $placement = 'APPEND')
	{
		if (is_array($script)) {
			if ($placement == 'PREPEND') {
				$script = array_reverse($script);
			}
			foreach ($script as $value) {
				$this->addJs($value, $mode, $placement);
			}
		}
		elseif (is_string($script)) {
			$this->view->headScript($mode, $script, $placement);
		}
	}
	
	/**
	 * Generates head section elements.
	 *
	 * @return string
	 */
	public function render()
	{
		$output = '';
		
		if ($this->view->headTitle()->getContainer()->count() > 0) {
			$output .= $this->view->headTitle()->toString();
		}
		
		$contentType = ($this->appendContentType) ? "\n\n"  . $this->view->headMeta()->appendHttpEquiv('Content-Type', 'text/html; charset=UTF-8')->toString($this->indent) : '';
		$output .= $contentType;
		
		if ($this->view->headLink()->getContainer()->count() > 0) {
			$output .= "\n\n";
			$output .= $this->view->headLink()->toString($this->indent);
		}
		
		if ($this->view->headStyle()->getContainer()->count() > 0) {
			$output .= "\n\n";
			$output .= $this->view->headStyle()->toString($this->indent);
		}
		
		if ($this->view->headScript()->getContainer()->count() > 0) {
			$output .= "\n\n";
			$output .= $this->view->headScript()->toString($this->indent);
		}
		
		
		return $output;
	}
}
?>